import logging
import gzip
import urllib.request
from urllib.error import (HTTPError, URLError)

import pybiomart
from scout.constants import CHROMOSOMES

LOG = logging.getLogger(__name__)

HPO_URL = ("http://compbio.charite.de/jenkins/job/hpo.annotations.monthly/"
           "lastStableBuild/artifact/annotation/{0}")
    

def get_request(url):
    """Return a requests response from url
    
    Args:
        url(str)
    
    Returns:
        decoded_data(str): Decoded response
    """
    try:
        LOG.info("Requesting %s", url)
        response = urllib.request.urlopen(url)
        if url.endswith('.gz'):
            LOG.info("Decompress zipped file")
            data = gzip.decompress(response.read())      # a `bytes` object
        else:
            data = response.read()      # a `bytes` object
        decoded_data = data.decode('utf-8')
    except HTTPError as err:
        LOG.warning("Something went wrong, perhaps the api key is not valid?")
        raise err
    except URLError as err:
        LOG.warning("Something went wrong, are you connected to internet?")
        raise err
    
    if 'Error' in decoded_data:
        raise URLError("Seems like url {} does not exist".format(url))
    
    return decoded_data
    

def fetch_resource(url):
    """Fetch a resource and return the resulting lines in a list
    Send file_name to get more clean log messages
    
    Args:
        url(str)
    
    Returns:
        lines(list(str))
    """
    try:
        data = get_request(url)
        lines = data.split('\n')
    except Exception as err:
        raise err
    
    return lines

def fetch_mim_files(api_key, mim2genes=False, mimtitles=False, morbidmap=False, genemap2=False):
    """Fetch the necessary mim files using a api key
    
    Args:
        api_key(str): A api key necessary to fetch mim data
    
    Returns:
        mim_files(dict): A dictionary with the neccesary files
    """

    LOG.info("Fetching OMIM files from https://omim.org/")
    mim2genes_url =  'https://omim.org/static/omim/data/mim2gene.txt'
    mimtitles_url= 'https://data.omim.org/downloads/{0}/mimTitles.txt'.format(api_key)
    morbidmap_url = 'https://data.omim.org/downloads/{0}/morbidmap.txt'.format(api_key)
    genemap2_url =  'https://data.omim.org/downloads/{0}/genemap2.txt'.format(api_key)
        
    mim_files = {}
    mim_urls = {}
    
    if mim2genes is True:
        mim_urls['mim2genes'] = mim2genes_url
    if mimtitles is True:
        mim_urls['mimtitles'] = mimtitles_url
    if morbidmap is True:
        mim_urls['morbidmap'] = morbidmap_url
    if genemap2 is True:
        mim_urls['genemap2'] = genemap2_url

    for file_name in mim_urls:
        url = mim_urls[file_name]
        mim_files[file_name] = fetch_resource(url)

    return mim_files

def fetch_hpo_terms():
    """Fetch the latest version of the hpo terms in .obo format
    
    Returns:
        res(list(str)): A list with the lines
    """
    url = "http://purl.obolibrary.org/obo/hp.obo"
    
    return fetch_resource(url)
    
def fetch_hpo_to_genes():
    """Fetch the latest version of the map from phenotypes to genes
    
    Returns:
        res(list(str)): A list with the lines
    
    """
    file_name = "ALL_SOURCES_ALL_FREQUENCIES_phenotype_to_genes.txt"
    url = HPO_URL.format(file_name)
    
    return fetch_resource(url)

def fetch_hpo_genes():
    """Fetch the latest version of the map from genes to hpo terms
    
    Returns:
        res(list(str)): A list with the lines
    
    """
    file_name = "ALL_SOURCES_ALL_FREQUENCIES_genes_to_phenotype.txt"
    url = HPO_URL.format(file_name)
    
    return fetch_resource(url)

def fetch_hpo_phenotype_to_terms():
    """Fetch the latest version of the map from phenotype to terms
    
    Returns:
        res(list(str)): A list with the lines
    
    """
    file_name = "ALL_SOURCES_ALL_FREQUENCIES_diseases_to_genes_to_phenotypes.txt"
    url = HPO_URL.format(file_name)
    
    return fetch_resource(url)

def fetch_ensembl_genes(build='37'):
    """Fetch the ensembl genes
    
    Args:
        build(str): ['37', '38']
    """
    if build == '37':
        url = 'http://grch37.ensembl.org'
    else:
        url = 'http://www.ensembl.org'
    
    LOG.info("Fetching ensembl genes from %s", url)
    dataset_name = 'hsapiens_gene_ensembl'
    
    dataset = pybiomart.Dataset(name=dataset_name, host=url)
    
    attributes = [
        'chromosome_name',
        'start_position',
        'end_position',
        'ensembl_gene_id',
        'hgnc_symbol',
        'hgnc_id',
    ]
    
    filters = {
        'chromosome_name': CHROMOSOMES,
    }
    
    result = dataset.query(
        attributes = attributes,
        filters = filters,
        use_attr_names=True,
    )
    
    return result

def fetch_ensembl_transcripts(build='37', chromosomes=None):
    """Fetch the ensembl genes
    
    Args:
        build(str): ['37', '38']
        chromosomes(iterable(str))
    
    Returns:
        result(DataFrame)
    """
    chromosomes = chromosomes or CHROMOSOMES
    LOG.info("Fetching ensembl transcripts build %s ...", build)
    if build == '37':
        url = 'http://grch37.ensembl.org'
    else:
        url = 'http://www.ensembl.org'
    
    dataset_name = 'hsapiens_gene_ensembl'
    
    dataset = pybiomart.Dataset(name=dataset_name, host=url)
    
    attributes = [
        'chromosome_name',
        'ensembl_gene_id',
        'ensembl_transcript_id',
        'transcript_start',
        'transcript_end',
        'refseq_mrna',
		'refseq_mrna_predicted',
		'refseq_ncrna',
    ]
    
    filters = {
        'chromosome_name': chromosomes,
    }
    
    result = dataset.query(
        attributes = attributes,
        filters = filters,
        use_attr_names=True,
    )
    
    return result

def fetch_ensembl_exons(build='37'):
    """Fetch the ensembl genes
    
    Args:
        build(str): ['37', '38']
    """
    LOG.info("Fetching ensembl exons build %s ...", build)
    if build == '37':
        url = 'http://grch37.ensembl.org'
    else:
        url = 'http://www.ensembl.org'
    
    dataset_name = 'hsapiens_gene_ensembl'
    
    dataset = pybiomart.Dataset(name=dataset_name, host=url)
    
    attributes = [
        'chromosome_name',
        'ensembl_gene_id',
        'ensembl_transcript_id',
        'ensembl_exon_id',
        'exon_chrom_start',
        'exon_chrom_end',
        '5_utr_start',
        '5_utr_end',
        '3_utr_start',
        '3_utr_end',
        'strand',
        'rank'
    ]
    
    filters = {
        'chromosome_name': CHROMOSOMES,
    }
    
    result = dataset.query(
        attributes = attributes,
        filters = filters
    )
    
    return result

def fetch_hgnc():
    """Fetch the hgnc genes file from 
        ftp://ftp.ebi.ac.uk/pub/databases/genenames/new/tsv/hgnc_complete_set.txt
    
    Returns:
        hgnc_gene_lines(list(str))
    """
    file_name = "hgnc_complete_set.txt"
    url = 'ftp://ftp.ebi.ac.uk/pub/databases/genenames/new/tsv/{0}'.format(file_name)
    LOG.info("Fetching HGNC genes")
    
    hgnc_lines = fetch_resource(url)
    
    return hgnc_lines

def fetch_exac_constraint():
    """Fetch the file with exac constraint scores
    
    Returns:
        exac_lines(iterable(str))
    """
    file_name = 'fordist_cleaned_exac_r03_march16_z_pli_rec_null_data.txt' 
    url = ('ftp://ftp.broadinstitute.org/pub/ExAC_release/release0.3/functional_gene_constraint'
           '/{0}').format(file_name)
    
    LOG.info("Fetching ExAC genes")
    
    try:
        exac_lines = fetch_resource(url)
    except URLError as err:
        LOG.info("Failed to fetch exac constraint scores file from ftp server")
        LOG.info("Try to fetch from google bucket...")
        url = ("https://storage.googleapis.com/gnomad-public/legacy/exacv1_downloads/release0.3.1"
               "/manuscript_data/forweb_cleaned_exac_r03_march16_z_data_pLI.txt.gz")

    exac_lines = fetch_resource(url)

    return exac_lines

def fetch_hpo_files(hpogenes=False, hpoterms=False, phenotype_to_terms=False, hpodisease=False):
    """Fetch the necessary mim files using a api key
    
    Args:
        api_key(str): A api key necessary to fetch mim data
    
    Returns:
        mim_files(dict): A dictionary with the neccesary files
    """

    LOG.info("Fetching HPO information from http://compbio.charite.de")
    base_url = ('http://compbio.charite.de/jenkins/job/hpo.annotations.monthly/'
                'lastStableBuild/artifact/annotation/{}')
    hpogenes_url =  base_url.format('ALL_SOURCES_ALL_FREQUENCIES_genes_to_phenotype.txt')
    hpoterms_url= base_url.format('ALL_SOURCES_ALL_FREQUENCIES_phenotype_to_genes.txt')
    hpo_phenotype_to_terms_url = base_url.format('ALL_SOURCES_ALL_FREQUENCIES_diseases_to_genes_to_phenotypes.txt')
    hpodisease_url = base_url.format('diseases_to_genes.txt')
        
    hpo_files = {}
    hpo_urls = {}
    
    if hpogenes is True:
        hpo_urls['hpogenes'] = hpogenes_url
    if hpoterms is True:
        hpo_urls['hpoterms'] = hpoterms_url
    if phenotype_to_terms is True:
        hpo_urls['phenotype_to_terms'] = hpo_phenotype_to_terms_url
    if hpodisease is True:
        hpo_urls['hpodisease'] = hpodisease_url

    for file_name in hpo_urls:
        url = hpo_urls[file_name]
        hpo_files[file_name] = request_file(url)

    return hpo_files




import pytest

from scout.exceptions import IntegrityError

def test_add_panel(gene_database, panel_obj):
    adapter = gene_database
    ## GIVEN a adapter with genes but no panels
    assert adapter.all_genes().count() > 0
    assert adapter.gene_panels().count() == 0
    ## WHEN inserting a panel
    adapter.add_gene_panel(panel_obj)
    ## THEN assert that the panel was loaded
    assert adapter.panel_collection.find().count() == 1

def test_add_same_panel_twice(gene_database, panel_obj):
    adapter = gene_database
    ## GIVEN a adapter with genes but no panels
    assert adapter.gene_panels().count() == 0
    ## WHEN inserting a panel twice
    adapter.add_gene_panel(panel_obj)
    ## THEN assert that IntegrityError is raised
    with pytest.raises(IntegrityError):
        adapter.add_gene_panel(panel_obj)

def test_get_panel_by_version(panel_database, panel_info):
    adapter = panel_database
    ## GIVEN a adapter with one gene panel
    res = adapter.gene_panels()
    assert res.count() == 1
    ## WHEN getting a panel
    res = adapter.gene_panel(
        panel_id=panel_info['panel_name'],
        version=float(panel_info['version']))
    ## THEN assert that the panel was loaded
    assert res['panel_name'] == panel_info['panel_name']

def test_get_panel_by_name(panel_database, panel_info):
    adapter = panel_database
    ## GIVEN a adapter with one gene panel
    res = adapter.gene_panels()
    assert res.count() == 1
    ## WHEN getting a panel
    res = adapter.gene_panel(panel_id=panel_info['panel_name'])
    ## THEN assert that the panel was loaded
    assert res['panel_name'] == panel_info['panel_name']

def test_get_non_existing_panel(panel_database, panel_info):
    adapter = panel_database
    ## GIVEN a adapter with one gene panel
    res = adapter.gene_panels()
    assert res.count() == 1
    ## WHEN getting a panel
    res = adapter.gene_panel(panel_id='non existing')
    ## THEN assert that the panel was loaded
    assert res is None

def test_get_panel_multiple_versions(panel_database, panel_obj):
    adapter = panel_database
    panel_obj['version'] = 2.0
    adapter.add_gene_panel(panel_obj)
    ## GIVEN a adapter with two gene panel
    res = adapter.gene_panels()
    assert res.count() == 2
    ## WHEN getting a panel
    res = adapter.gene_panel(panel_id=panel_obj['panel_name'])
    ## THEN assert that the last version is fetched
    assert res['version'] == 2.0

def test_add_pending(panel_database):
    adapter = panel_database
    panel_obj = adapter.panel_collection.find_one()
    hgnc_obj = adapter.hgnc_collection.find_one()
    ## GIVEN an adapter with a gene panel
    res = adapter.gene_panels()
    assert res.count() == 1
    ## WHEN adding a pending action
    res = adapter.add_pending(
        panel_obj=panel_obj,
        hgnc_gene=hgnc_obj,
        action='add'
    )
    ## THEN assert that the last version is fetched
    assert len(res['pending']) == 1

def test_add_pending_wrong_action(panel_database):
    adapter = panel_database
    panel_obj = adapter.panel_collection.find_one()
    hgnc_obj = adapter.hgnc_collection.find_one()
    ## GIVEN an adapter with a gene panel
    res = adapter.gene_panels()
    assert res.count() == 1
    ## WHEN adding a pending action with invalid action
    with pytest.raises(ValueError):
    ## THEN assert that an error is raised
        res = adapter.add_pending(
            panel_obj=panel_obj,
            hgnc_gene=hgnc_obj,
            action='hello'
        )

def test_update_panel_panel_name(panel_database):
    adapter = panel_database
    panel_obj = adapter.panel_collection.find_one()
    old_name = panel_obj['panel_name']
    new_name = 'new name'
    ## GIVEN an adapter with a gene panel
    res = adapter.gene_panels()
    assert res.count() == 1
    ## WHEN updating the panel name
    panel_obj['panel_name'] = new_name

    res = adapter.update_panel(panel_obj)

    ## THEN assert that the last version is fetched
    assert res['panel_name'] == new_name

def test_apply_pending_delete_gene(panel_database):
    ## GIVEN an adapter with a gene panel
    adapter = panel_database
    panel_obj = adapter.panel_collection.find_one()

    gene = panel_obj['genes'][0]
    hgnc_id = gene['hgnc_id']
    hgnc_symbol = gene['symbol']

    action = {
        'hgnc_id': hgnc_id,
        'action': 'delete',
        'symbol': hgnc_symbol,
        'info': {}
    }
    ## WHEN adding a action to the panel
    panel_obj['pending'] = [action]
    old_version = panel_obj['version']

    updated_panel_id = adapter.apply_pending(panel_obj, panel_obj['version']+1)
    updated_panel = adapter.panel_collection.find_one( {'_id' : updated_panel_id} )

    # assert that the updated panel has a newer version
    assert updated_panel['version'] != old_version

    ## THEN assert that the new panel does not have the deleted gene
    for gene in updated_panel['genes']:
        assert gene['hgnc_id'] != hgnc_id

def test_apply_pending_delete_two_genes(real_panel_database):
    adapter = real_panel_database
    panel_obj = adapter.panel_collection.find_one()

    gene = panel_obj['genes'][0]
    gene2 = panel_obj['genes'][1]

    hgnc_ids = [gene['hgnc_id'], gene2['hgnc_id']]

    action = {
        'hgnc_id': gene['hgnc_id'],
        'action': 'delete',
        'symbol': gene['symbol'],
        'info': {}
    }

    action2 = {
        'hgnc_id': gene2['hgnc_id'],
        'action': 'delete',
        'symbol': gene2['symbol'],
        'info': {}
    }

    panel_obj['pending'] = [action, action2]

    updated_panel_id = adapter.apply_pending(panel_obj, panel_obj['version'])
    updated_panel = adapter.panel_collection.find_one( {'_id' : updated_panel_id} )

    for gene in updated_panel['genes']:
        assert gene['hgnc_id'] not in hgnc_ids

def test_apply_pending_add_gene(real_panel_database):
    adapter = real_panel_database
    panel_obj = adapter.panel_collection.find_one()

    gene = panel_obj['genes'][0]
    hgnc_id = gene['hgnc_id']
    hgnc_symbol = gene['symbol']

    panel_obj['genes'] = []
    adapter.update_panel(panel_obj)

    panel_obj = adapter.panel_collection.find_one()
    assert len(panel_obj['genes']) == 0

    action = {
        'hgnc_id': hgnc_id,
        'action': 'add',
        'symbol': hgnc_symbol,
        'info': {}
    }

    panel_obj['pending'] = [action]

    # update panel version to panel_version +1
    updated_panel_id = adapter.apply_pending(panel_obj, panel_obj['version']+1)
    updated_panel = adapter.panel_collection.find_one( {'_id' : updated_panel_id} )

    #assert that panel version was updated
    assert updated_panel['version'] == panel_obj['version'] + 1

    assert len(updated_panel['genes']) == 1

def test_apply_pending_add_two_genes(real_panel_database):
    adapter = real_panel_database
    panel_obj = adapter.panel_collection.find_one()

    gene = panel_obj['genes'][0]
    gene2 = panel_obj['genes'][1]
    hgnc_ids = [gene['hgnc_id'], gene['hgnc_id']]
    hgnc_symbols = [gene['symbol'], gene['symbol']]

    panel_obj['genes'] = []
    adapter.update_panel(panel_obj)

    panel_obj = adapter.panel_collection.find_one()
    assert len(panel_obj['genes']) == 0

    action1 = {
        'hgnc_id': hgnc_ids[0],
        'action': 'add',
        'symbol': hgnc_symbols[0],
        'info': {}
    }

    action2 = {
        'hgnc_id': hgnc_ids[1],
        'action': 'add',
        'symbol': hgnc_symbols[1],
        'info': {}
    }

    panel_obj['pending'] = [action1, action2]

    # update panel without changing panel version
    updated_panel_id = adapter.apply_pending(panel_obj, panel_obj['version'])
    updated_panel = adapter.panel_collection.find_one( {'_id' : updated_panel_id} )

    #assert that panel version was NOT updated
    assert updated_panel['version'] == panel_obj['version']

    assert len(updated_panel['genes']) == 2
    for gene in updated_panel['genes']:
        assert gene['hgnc_id'] in hgnc_ids



# -*- coding: utf-8 -*-
import logging

from datetime import datetime as datetime

from scout.exceptions import IntegrityError

LOG = logging.getLogger(__name__)

def build_gene(gene_info, adapter):
    """Build a panel_gene object
    
    Args:
        gene_info(dict)
    
    Returns:
        gene_obj(dict)
    
        panel_gene = dict(
            hgnc_id = int, # required
            symbol = str, 

            disease_associated_transcripts = list, # list of strings that represent refseq transcripts
            reduced_penetrance = bool, 
            mosaicism = bool, 
            database_entry_version = str,

            ar = bool,
            ad = bool,
            mt = bool,
            xr = bool,
            xd = bool,
            x = bool,
            y = bool,
        )

    """
    symbol = gene_info.get('hgnc_symbol')
    try:
        # A gene has to have a hgnc id
        hgnc_id = gene_info['hgnc_id']
        if not hgnc_id:
            raise KeyError()
        gene_obj = dict(hgnc_id=hgnc_id)
    except KeyError as err:
        raise KeyError("Gene {0} is missing hgnc id. Panel genes has to have hgnc_id".format(symbol))
    
    hgnc_gene = adapter.hgnc_gene(hgnc_id)
    if hgnc_gene is None:
        raise IntegrityError("hgnc_id {0} is not in the gene database!".format(hgnc_id))
    
    gene_obj['symbol'] = hgnc_gene['hgnc_symbol']
    if symbol != gene_obj['symbol']:
        LOG.warning("Symbol in database does not correspond to symbol in panel file for gene %s", hgnc_id)
        LOG.warning("Using symbol %s for gene %s, instead of %s" % 
                    (hgnc_gene['hgnc_symbol'], hgnc_id, symbol))

    if gene_info.get('transcripts'):
        gene_obj['disease_associated_transcripts'] = gene_info['transcripts']
    
    if gene_info.get('reduced_penetrance'):
        gene_obj['reduced_penetrance'] = True
    
    if gene_info.get('mosaicism'):
        gene_obj['mosaicism'] = True
    
    if gene_info.get('database_entry_version'):
        gene_obj['database_entry_version'] = gene_info['database_entry_version']
    
    if gene_info.get('inheritance_models'):
        for model in gene_info['inheritance_models']:
            if model == 'AR':
                gene_obj['ar'] = True
            if model == 'AD':
                gene_obj['ad'] = True
            if model == 'MT':
                gene_obj['mt'] = True
            if model == 'XR':
                gene_obj['xr'] = True
            if model == 'XD':
                gene_obj['xd'] = True
            if model == 'X':
                gene_obj['x'] = True
            if model == 'Y':
                gene_obj['y'] = True

    return gene_obj


def build_panel(panel_info, adapter):
    """Build a gene_panel object

        Args:
            panel_info(dict): A dictionary with panel information
            adapter (scout.adapter.MongoAdapter)

        Returns:
            panel_obj(dict)

    gene_panel = dict(
        panel_id = str, # required
        institute = str, # institute_id, required
        version = float, # required
        date = datetime, # required
        display_name = str, # default is panel_name
        genes = list, # list of panel genes, sorted on panel_gene['symbol']
    )

    """
    
    panel_name = panel_info.get('panel_id', panel_info.get('panel_name'))
    if not panel_name:
        raise KeyError("Panel has to have a id")
    
    panel_obj = dict(panel_name = panel_name)
    LOG.info("Building panel with name: {0}".format(panel_name))

    try:
        institute_id = panel_info['institute']
    except KeyError as err:
        raise KeyError("Panel has to have a institute")

    # Check if institute exists in database
    if adapter.institute(institute_id) is None:
        raise IntegrityError("Institute %s could not be found" % institute_id)
    
    panel_obj['institute'] = panel_info['institute']
    
    panel_obj['version'] = float(panel_info['version'])
    
    try:
        panel_obj['date'] = panel_info['date']
    except KeyError as err:
        raise KeyError("Panel has to have a date")


    panel_obj['display_name'] = panel_info.get('display_name', panel_obj['panel_name'])
    
    gene_objs = []
    fail = False
    for gene_info in panel_info.get('genes', []):
        try:
            gene_obj = build_gene(gene_info, adapter)
            gene_objs.append(gene_obj)
        except IntegrityError as err:
            LOG.warning(err)
            fail=True
    
    if fail:
        raise IntegrityError("Some genes did not exist in database. Please see log messages.")

    panel_obj['genes'] = gene_objs

    return panel_obj



from __future__ import unicode_literals

from bson.objectid import ObjectId

class Exon(dict):
    """Exon dictionary
    
        "exon_id": str, # str(chrom-start-end)
        "chrom": str, 
        "start": int, 
        "end": int,     
        "transcript": str, # ENST ID
        "hgnc_id": int,      # HGNC_id
        "rank": int, # Order of exon in transcript
        "build": str, # Genome build
    
    """
    def __init__(self, exon_id, chrom, start, end, transcript, hgnc_id, rank, build='37'):
        super(Exon, self).__init__()
        self['exon_id'] = exon_id
        self['chrom'] = chrom
        self['start'] = int(start)
        self['end'] = int(end)
        self['transcript'] = transcript
        self['hgnc_id'] = hgnc_id
        self['rank'] = int(rank)
        self['build'] = build

class HgncTranscript(dict):
    """Transcript dictionary
    
        "transcript_id": str, # ensembl id
        "hgnc_id": int,
        "chrom": str, 
        "start": int, 
        "end": int, 
        "is_primary": bool, 
        "refseq_id": str, 
        "refseq_identifiers": list, 
        "build": str, # Genome build
        "length": int
    """
    def __init__(self, transcript_id, hgnc_id, chrom, start, end, is_primary=False, 
                 refseq_id=None, refseq_identifiers=None, build='37'):
        super(HgncTranscript, self).__init__()
        self['ensembl_transcript_id'] = transcript_id
        self['hgnc_id'] = int(hgnc_id)
        self['chrom'] = chrom
        self['start'] = int(start)
        self['end'] = int(end)
        self['is_primary'] = is_primary
        self['refseq_id'] = refseq_id
        self['refseq_identifiers'] = refseq_identifiers
        self['build'] = build
        self['length'] = self['end'] - self['start']

class HgncGene(dict):
    """HgncGene dictionary
    
        'hgnc_id': int, # This is the hgnc id, required:
        'hgnc_symbol': str, # The primary symbol, required 
        'ensembl_id': str, # required
        'build': str, # '37' or '38', defaults to '37', required
        
        'chromosome': str, # required
        'start': int, # required
        'end': int, # required
        
        'description': str, # Gene description
        'aliases': list(), # Gene symbol aliases, includes hgnc_symbol, str
        'entrez_id': int,
        'omim_id': int,
        'pli_score': float,
        'primary_transcripts': list(), # List of refseq transcripts (str)
        'ucsc_id': str,
        'uniprot_ids': list(), # List of str
        'vega_id': str,
        
        # Inheritance information
        'inheritance_models': list(), # List of model names
        'incomplete_penetrance': bool, # Acquired from HPO
        
        # Phenotype information
        'phenotypes': list(), # List of dictionaries with phenotype information
    """
    def __init__(self, hgnc_id, hgnc_symbol, ensembl_id, chrom, start, end, 
                 description=None, aliases=None, entrez_id=None, omim_id=None, 
                 pli_score=None, primary_transcripts=None, ucsc_id=None, 
                 uniprot_ids=None, vega_id=None, inheritance_models=None, 
                 incomplete_penetrance=False, phenotypes=None, build='37'):
        super(HgncGene, self).__init__()
        self['hgnc_id'] = int(hgnc_id)
        self['hgnc_symbol'] = hgnc_symbol
        self['ensembl_id'] = ensembl_id

        self['chromosome'] = chrom
        self['start'] = int(start)
        self['end'] = int(end)
        self['length'] = self['end'] - self['start']

        self['description'] = description
        self['aliases'] = aliases
        self['primary_transcripts'] = primary_transcripts
        self['inheritance_models'] = inheritance_models
        self['phenotypes'] = phenotypes

        self['entrez_id'] = entrez_id
        if entrez_id:
            self['entrez_id'] = int(entrez_id)
        
        self['omim_id'] = omim_id
        if omim_id:
            self['omim_id'] = int(omim_id)

        self['ucsc_id'] = ucsc_id
        self['uniprot_ids'] = uniprot_ids
        self['vega_id'] = vega_id
        
        self['pli_score'] = pli_score
        if pli_score:
            self['pli_score'] = float(pli_score)
        
        self['incomplete_penetrance'] = incomplete_penetrance

        self['build'] = build



import logging
from pprint import pprint as pp

import click

LOG = logging.getLogger(__name__)

@click.command(short_help='Upload variants to existing case')
@click.argument('case-id')
@click.option('-i', '--institute', help='institute id of related cases')
@click.option('-f', '--force', is_flag=True, help='upload without request')
@click.option('--cancer', is_flag=True, help='Upload clinical cancer variants')
@click.option('--cancer-research', is_flag=True, help='Upload research cancer variants')
@click.option('--sv', is_flag=True, help='Upload clinical structural variants')
@click.option('--sv-research', is_flag=True, help='Upload research structural variants')
@click.option('--snv', is_flag=True, help='Upload clinical SNV variants')
@click.option('--snv-research', is_flag=True, help='Upload research SNV variants')
@click.option('--str-clinical', is_flag=True, help='Upload clinical STR variants')
@click.option('--chrom', help='If region, specify the chromosome')
@click.option('--start', type=int, help='If region, specify the start')
@click.option('--end', type=int, help='If region, specify the end')
@click.option('--hgnc-id', type=int, help='If all variants from a gene, specify the gene id')
@click.option('--hgnc-symbol', help='If all variants from a gene, specify the gene symbol')
@click.option('--rank-treshold', default=5, help='Specify the rank score treshold',
                show_default=True)
@click.pass_context
def variants(context, case_id, institute, force, cancer, cancer_research, sv, 
             sv_research, snv, snv_research, str_clinical, chrom, start, end, hgnc_id, 
             hgnc_symbol, rank_treshold):
    """Upload variants to a case

        Note that the files has to be linked with the case, 
        if they are not use 'scout update case'.
    """
    LOG.info("Running scout load variants")
    adapter = context.obj['adapter']
    
    if institute:
        case_id = "{0}-{1}".format(institute, case_id)
    else:
        institute = case_id.split('-')[0]
    case_obj = adapter.case(case_id=case_id)
    if case_obj is None:
        LOG.info("No matching case found")
        context.abort()

    files = [
        {'category': 'cancer', 'variant_type': 'clinical', 'upload': cancer},
        {'category': 'cancer', 'variant_type': 'research', 'upload': cancer_research},
        {'category': 'sv', 'variant_type': 'clinical', 'upload': sv},
        {'category': 'sv', 'variant_type': 'research', 'upload': sv_research},
        {'category': 'snv', 'variant_type': 'clinical', 'upload': snv},
        {'category': 'snv', 'variant_type': 'research', 'upload': snv_research},
        {'category': 'str', 'variant_type': 'clinical', 'upload': str_clinical},
    ]
    
    gene_obj = None
    if (hgnc_id or hgnc_symbol):
        if hgnc_id:
            gene_obj = adapter.hgnc_gene(hgnc_id)
        if hgnc_symbol:
            for res in adapter.gene_by_alias(hgnc_symbol):
                gene_obj = res
        if not gene_obj:
            LOG.warning("The gene could not be found")
            context.abort()
        
    i = 0
    for file_type in files:
        variant_type = file_type['variant_type']
        category = file_type['category']
        
        if file_type['upload']:
            i += 1
            if variant_type == 'research':
                if not (force or case_obj['research_requested']):
                    LOG.warn("research not requested, use '--force'")
                    context.abort()
            
            LOG.info("Delete {0} {1} variants for case {2}".format(
                         variant_type, category, case_id))
            adapter.delete_variants(case_id=case_obj['_id'], 
                                    variant_type=variant_type,
                                    category=category)
            
            LOG.info("Load {0} {1} variants for case {2}".format(
                         variant_type, category, case_id))
            
            try:
                adapter.load_variants(
                    case_obj=case_obj, 
                    variant_type=variant_type, 
                    category=category,
                    rank_threshold=rank_treshold, 
                    chrom=chrom, 
                    start=start, 
                    end=end,
                    gene_obj=gene_obj
                )
            except Exception as e:
                LOG.warning(e)
                context.abort()
    if i == 0:
        LOG.info("No files where specified to upload variants from")



from pprint import pprint as pp

from scout.constants import (CHROMOSOMES, CHROMOSOME_INTEGERS)

def export_variants(adapter, collaborator, document_id=None):
    """Export causative variants for a collaborator
    
    Args:
        adapter(MongoAdapter)
        collaborator(str)
        document_id(str): Search for a specific variant
    
    Yields:
        variant_obj(scout.Models.Variant): Variants marked as causative ordered by position. 
    """
    
    # Store the variants in a list for sorting
    variants = []
    if document_id:
        yield adapter.variant(document_id)
        return
    
    ##TODO add check so that same variant is not included more than once
    for document_id in adapter.get_causatives(institute_id=collaborator):
        variant_obj = adapter.variant(document_id)
        
        chrom = variant_obj['chromosome']
        # Convert chromosome to integer for sorting
        chrom_int = CHROMOSOME_INTEGERS.get(chrom)
        if not chrom_int:
            LOG.info("Unknown chromosome %s", chrom)
            continue
        
        # Add chromosome and position to prepare for sorting
        variants.append((chrom_int, variant_obj['position'], variant_obj))
    
    # Sort varants based on position
    variants.sort(key=lambda x: (x[0], x[1]))
    
    for variant in variants:
        variant_obj = variant[2]
        yield variant_obj
    


import logging
import click
import urllib.request

from pprint import pprint as pp

from scout.load.hpo import load_hpo_terms
from scout.utils.requests import (fetch_hpo_terms, fetch_hpo_to_genes)

from scout.commands.utils import abort_if_false

LOG = logging.getLogger(__name__)

@click.command('hpo', short_help='Update hpo terms')
@click.option('--yes', is_flag=True, callback=abort_if_false,
              expose_value=False,
              prompt='Are you sure you want to drop the hpo terms?')
@click.pass_context
def hpo(context):
    """
    Update the hpo terms in the database. Fetch the latest release and update terms.
    """
    LOG.info("Running scout update hpo")
    adapter = context.obj['adapter']

    LOG.info("Dropping HPO terms")
    adapter.hpo_term_collection.drop()
    LOG.debug("HPO terms dropped")

    load_hpo_terms(adapter)



# -*- coding: utf-8 -*-
import logging

import click

from scout.utils.handle import get_file_handle
from scout.parse.panel import parse_genes

log = logging.getLogger(__name__)


@click.command('convert', short_help='Convert gene panels')
@click.argument('panel', type=click.File('r'))
@click.pass_context
def convert(context, panel):
    """Convert a gene panel with hgnc symbols to a new one with hgnc ids."""
    adapter = context.obj['adapter']
    new_header = ["hgnc_id","hgnc_symbol","disease_associated_transcripts",
                 "reduced_penetrance", "genetic_disease_models", "mosaicism",
                 "database_entry_version"]
    
    genes = parse_genes(panel)
    
    adapter.add_hgnc_id(genes)
    
    click.echo("#{0}".format('\t'.join(new_header)))
    for gene in genes:
        if gene.get('hgnc_id'):
            print_info = []
            for head in new_header:
                print_info.append(str(gene[head]) if gene.get(head) else '')
            
            click.echo('\t'.join(print_info))


import logging

import click

LOG = logging.getLogger(__name__)

@click.command('case', short_help='Update a case')
@click.argument('case_id', required=False)
@click.option('--case-name', '-n',
                help="Add/update the display name of case",
)
@click.option('--institute', '-i',
                help="Update what institutes that has access to case",
)
@click.option('--collaborator', '-c',
                help="Add a collaborator to the case",
)
@click.option('--vcf', type=click.Path(exists=True),
              help='path to clinical VCF file to be added')
@click.option('--vcf-sv', type=click.Path(exists=True),
              help='path to clinical SV VCF file to be added')
@click.option('--vcf-cancer', type=click.Path(exists=True),
              help='path to clinical cancer VCF file to be added')
@click.option('--vcf-research', type=click.Path(exists=True),
              help='path to research VCF file to be added')
@click.option('--vcf-sv-research', type=click.Path(exists=True),
              help='path to research VCF with SV variants to be added')
@click.option('--vcf-cancer-research', type=click.Path(exists=True),
              help='path to research VCF with cancer variants to be added')
@click.option('--peddy-ped', type=click.Path(exists=True),
              help='path to outfile .peddy.ped from peddy')
@click.pass_context
def case(context, case_id, case_name, institute, collaborator, vcf, vcf_sv,
         vcf_cancer, vcf_research, vcf_sv_research, vcf_cancer_research, peddy_ped):
    """
    Update a case in the database
    """
    adapter = context.obj['adapter']
    if not case_id:
        if not (case_name and institute):
            LOG.info("Please specify which case to update.")
            context.abort
        case_id = "{0}-{1}".format(institute, case_name)
    # Chock if the case exists
    case_obj = adapter.case(case_id)
    
    if not case_obj:
        LOG.warning("Case %s could not be found", case_id)
        context.abort()
    
    if collaborator:
        if not adapter.institute(collaborator):
            LOG.warning("Institute %s could not be found", collaborator)
            context.abort()
        if not collaborator in case_obj['collaborators']:
            case_obj['collaborators'].append(collaborator)
            LOG.info("Adding collaborator %s", collaborator)
    
    if vcf:
        LOG.info("Updating 'vcf_snv' to %s", vcf)
        case_obj['vcf_files']['vcf_snv'] = vcf
    if vcf_sv:
        LOG.info("Updating 'vcf_sv' to %s", vcf_sv)
        case_obj['vcf_files']['vcf_sv'] = vcf_sv
    if vcf_cancer:
        LOG.info("Updating 'vcf_cancer' to %s", vcf_cancer)
        case_obj['vcf_files']['vcf_cancer'] = vcf_cancer
    if vcf_research:
        LOG.info("Updating 'vcf_research' to %s", vcf_research)
        case_obj['vcf_files']['vcf_research'] = vcf_research
    if vcf_sv_research:
        LOG.info("Updating 'vcf_sv_research' to %s", vcf_sv_research)
        case_obj['vcf_files']['vcf_sv_research'] = vcf_sv_research
    if vcf_cancer_research:
        LOG.info("Updating 'vcf_cancer_research' to %s", vcf_cancer_research)
        case_obj['vcf_files']['vcf_cancer_research'] = vcf_cancer_research

    adapter.update_case(case_obj)



# -*- coding: utf-8 -*-
import decimal

from flask_wtf import FlaskForm
from wtforms import (BooleanField, DecimalField, Field, TextField, SelectMultipleField,
                     HiddenField, IntegerField, SubmitField)
from wtforms.widgets import TextInput
from flask_wtf.file import FileField

from scout.constants import (CLINSIG_MAP, FEATURE_TYPES, GENETIC_MODELS, SO_TERMS,
                             SPIDEX_LEVELS, SV_TYPES)

CLINSIG_OPTIONS = list(CLINSIG_MAP.items())
FUNC_ANNOTATIONS = [(term, term.replace('_', ' ')) for term in SO_TERMS]
REGION_ANNOTATIONS = [(term, term.replace('_', ' ')) for term in FEATURE_TYPES]
SV_TYPE_CHOICES = [(term, term.replace('_', ' ').upper()) for term in SV_TYPES]
SPIDEX_CHOICES = [(term, term.replace('_', ' ')) for term in SPIDEX_LEVELS]

class TagListField(Field):
    widget = TextInput()

    def _value(self):
        if self.data:
            return ', '.join(self.data)
        else:
            return ''

    def process_formdata(self, valuelist):
        if valuelist:
            self.data = [x.strip() for x in valuelist[0].split(',') if x.strip()]
        else:
            self.data = []


class BetterDecimalField(DecimalField):
    """
    Very similar to WTForms DecimalField, except handles ',' and '.'.
    """

    def process_formdata(self, valuelist):
        if valuelist:
            raw_decimal = valuelist[0].replace(',', '.')
            try:
                self.data = decimal.Decimal(raw_decimal)
            except (decimal.InvalidOperation, ValueError):
                self.data = None
                raise ValueError(self.gettext('Not a valid decimal value'))


class FiltersForm(FlaskForm):
    """Base FiltersForm for SNVs"""
    variant_type = HiddenField(default='clinical')
    gene_panels = SelectMultipleField(choices=[])
    hgnc_symbols = TagListField('HGNC Symbols/Ids (case sensitive)')

    symbol_file = FileField('Symbol File')

    region_annotations = SelectMultipleField(choices=REGION_ANNOTATIONS)
    functional_annotations = SelectMultipleField(choices=FUNC_ANNOTATIONS)
    genetic_models = SelectMultipleField(choices=GENETIC_MODELS)

    cadd_score = BetterDecimalField('CADD', places=2)
    cadd_inclusive = BooleanField('CADD inclusive')
    clinsig = SelectMultipleField('CLINSIG', choices=CLINSIG_OPTIONS)
    clinsig_confident_always_returned = BooleanField('CLINSIG Confident')
    spidex_human = SelectMultipleField('SPIDEX', choices=SPIDEX_CHOICES)

    gnomad_frequency = BetterDecimalField('gnomadAF', places=2)
    chrom = TextField('Chromosome')
    local_obs = IntegerField('Local obs. (archive)')

    filter_variants = SubmitField(label='Filter variants')
    clinical_filter = SubmitField(label='Clinical filter')

class CancerFiltersForm(FiltersForm):
    """docstring for CancerFiltersForm"""
    depth = IntegerField('Depth >')
    alt_count = IntegerField('Min alt count >')
    control_frequency = BetterDecimalField('Control freq. <', places=2)
    mvl_tag = BooleanField('In Managed Variant List')

class StrFiltersForm(FlaskForm):
    """docstring for CancerFiltersForm"""
    variant_type = HiddenField(default='clinical')

    chrom = TextField('Chromosome')
    gene_panels = SelectMultipleField(choices=[])
    repids = TagListField()

class SvFiltersForm(FiltersForm):
    """Extends FiltersForm for structural variants"""
    size = TextField('Length')
    size_inclusive = BooleanField('Length inclusive')
    svtype = SelectMultipleField('SVType', choices=SV_TYPE_CHOICES)

    clingen_ngi = IntegerField('ClinGen NGI obs')



import logging
import click

LOG = logging.getLogger(__name__)


@click.command('users', short_help='Display users')
@click.pass_context
def users(context):
    """Show all users in the database"""
    LOG.info("Running scout view users")
    adapter = context.obj['adapter']

    user_objs = adapter.users()
    if user_objs.count() == 0:
        LOG.info("No users found")
        context.abort()

    click.echo("#name\temail\troles\tinstitutes")
    for user_obj in user_objs:
        click.echo("{0}\t{1}\t{2}\t{3}\t".format(
            user_obj['name'],
            user_obj.get('mail', user_obj['_id']),
            ', '.join(user_obj.get('roles', [])),
            ', '.join(user_obj.get('institutes', [])),
        )
        )




def parse_frequencies(variant, transcripts):
    """Add the frequencies to a variant
    
    Frequencies are parsed either directly from keys in info fieds or from the 
    transcripts is they are annotated there.

    Args:
        variant(cyvcf2.Variant): A parsed vcf variant
        transcripts(iterable(dict)): Parsed transcripts

    Returns:
        frequencies(dict): A dictionary with the relevant frequencies
    """
    frequencies = {}
    # These lists could be extended...
    thousand_genomes_keys = ['1000GAF']
    thousand_genomes_max_keys = ['1000G_MAX_AF']

    exac_keys = ['EXACAF']
    exac_max_keys = ['ExAC_MAX_AF', 'EXAC_MAX_AF']

    gnomad_keys = ['GNOMADAF', 'GNOMAD_AF']
    gnomad_max_keys = ['GNOMADAF_POPMAX', 'GNOMADAF_MAX']
    
    for test_key in thousand_genomes_keys:
        thousand_g = parse_frequency(variant, test_key)
        if thousand_g:
            frequencies['thousand_g'] = thousand_g
            break

    for test_key in thousand_genomes_max_keys:
        thousand_g_max = parse_frequency(variant, test_key)
        if thousand_g_max:
            frequencies['thousand_g_max'] = thousand_g_max
            break

    for test_key in exac_keys:
        exac = parse_frequency(variant, test_key)
        if exac:
            frequencies['exac'] = exac
            break

    for test_key in exac_max_keys:
        exac_max = parse_frequency(variant, test_key)
        if exac_max:
            frequencies['exac_max'] = exac_max
            break

    for test_key in gnomad_keys:
        gnomad = parse_frequency(variant, test_key)
        if gnomad:
            frequencies['gnomad'] = gnomad
            break

    for test_key in gnomad_max_keys:
        gnomad_max = parse_frequency(variant, test_key)
        if gnomad_max:
            frequencies['gnomad_max'] = gnomad_max
            break

    # Search transcripts if not found in VCF
    if not frequencies:
        for transcript in transcripts:
            exac = transcript.get('exac_maf')
            exac_max = transcript.get('exac_max')
            
            thousand_g = transcript.get('thousand_g_maf')
            thousandg_max = transcript.get('thousandg_max')
            
            gnomad = transcript.get('gnomad_maf')
            gnomad_max = transcript.get('gnomad_max')
            if exac:
                frequencies['exac'] = exac
            if exac_max:
                frequencies['exac_max'] = exac_max
            if thousand_g:
                frequencies['thousand_g'] = thousand_g
            if thousandg_max:
                frequencies['thousand_g_max'] = thousandg_max
            if gnomad:
                frequencies['gnomad'] = gnomad
            if gnomad_max:
                frequencies['gnomad_max'] = gnomad_max

    #These are SV-specific frequencies
    thousand_g_left = parse_frequency(variant, 'left_1000GAF')
    if thousand_g_left:
        frequencies['thousand_g_left'] = thousand_g_left 
    
    thousand_g_right = parse_frequency(variant, 'right_1000GAF')
    if thousand_g_right:
        frequencies['thousand_g_right'] = thousand_g_right 

    return frequencies


def parse_frequency(variant, info_key):
    """Parse any frequency from the info dict
    
    Args:
        variant(cyvcf2.Variant)
        info_key(str)
    
    Returns:
        frequency(float): or None if frequency does not exist
    """
    raw_annotation = variant.INFO.get(info_key)
    raw_annotation = None if raw_annotation == '.' else raw_annotation
    frequency = float(raw_annotation) if raw_annotation else None
    return frequency

def parse_sv_frequencies(variant):
    """Parsing of some custom sv frequencies
    
    These are very specific at the moment, this will hopefully get better over time when the 
    field of structural variants is more developed.
    
    Args:
        variant(cyvcf2.Variant)
    
    Returns:
        sv_frequencies(dict)
    """
    frequency_keys = [
        'clingen_cgh_benignAF', 
        'clingen_cgh_benign',
        'clingen_cgh_pathogenicAF',
        'clingen_cgh_pathogenic',
        'clingen_ngi',
        'clingen_ngiAF',
        'decipherAF',
        'decipher'
    ]
    sv_frequencies = {}
    
    for key in frequency_keys:
        value = variant.INFO.get(key, 0)
        if 'AF' in key:
            value = float(value)
        else:
            value = int(value)
        if value > 0:
            sv_frequencies[key] = value
    
    return sv_frequencies
    


import logging

from datetime import datetime
from pprint import pprint as pp

import pymongo

from scout.exceptions import IntegrityError
from scout.constants import PHENOTYPE_GROUPS

LOG = logging.getLogger(__name__)


class InstituteHandler(object):

    def add_institute(self, institute_obj):
        """Add a institute to the database

            Args:
                institute_obj(Institute)
        """
        internal_id = institute_obj['internal_id']
        display_name = institute_obj['internal_id']

        # Check if institute already exists
        if self.institute(institute_id=internal_id):
            raise IntegrityError("Institute {0} already exists in database"
                                 .format(display_name))

        LOG.info("Adding institute with internal_id: {0} and "
                    "display_name: {1}".format(internal_id,
                                               display_name))

        insert_info = self.institute_collection.insert_one(institute_obj)
        ##TODO check if insert info was ok
        LOG.info("Institute saved")

    def update_institute(self, internal_id, sanger_recipient=None, coverage_cutoff=None, 
                         frequency_cutoff=None, display_name=None, remove_sanger=None,
                         phenotype_groups=None, group_abbreviations=None, add_groups=None):
        """Update the information for an institute

        Args:
            internal_id(str): The internal institute id
            sanger_recipient(str): Email adress to add for sanger order
            coverage_cutoff(int): Update coverage cutoff
            frequency_cutoff(float): New frequency cutoff
            display_name(str): New display name
            remove_sanger(str): Email adress for sanger user to be removed
            phenotype_groups(iterable(str)): New phenotype groups
            group_abbreviations(iterable(str))
            add_groups: If groups should be added. If False replace groups
        
        Returns:
            updated_institute(dict)
                     
        """
        add_groups = add_groups or False
        institute_obj = self.institute(internal_id)
        if not institute_obj:
            raise IntegrityError("Institute {} does not exist in database".format(internal_id))
        
        updates = {}
        updated_institute = institute_obj

        if sanger_recipient:
            user_obj = self.user(sanger_recipient)
            if not user_obj:
                raise IntegrityError("user {} does not exist in database".format(sanger_recipient))
                
            LOG.info("Updating sanger recipients for institute: {0} with {1}".format(
                     internal_id, sanger_recipient))
            updates['$push'] = {'sanger_recipients':remove_sanger}

        if remove_sanger:
            LOG.info("Removing sanger recipient {0} from institute: {1}".format(
                     remove_sanger, internal_id))
            updates['$pull'] = {'sanger_recipients':remove_sanger}

        if coverage_cutoff:
            LOG.info("Updating coverage cutoff for institute: {0} to {1}".format(
                            internal_id, coverage_cutoff))
            updates['$set'] = {'coverage_cutoff': coverage_cutoff}
        
        if frequency_cutoff:
            LOG.info("Updating frequency cutoff for institute: {0} to {1}".format(
                            internal_id, frequency_cutoff))
            if not '$set' in updates:
                updates['$set'] = {}
            updates['$set'] = {'frequency_cutoff': frequency_cutoff}

        if display_name:
            LOG.info("Updating display name for institute: {0} to {1}".format(
                            internal_id, display_name))
            if not '$set' in updates:
                updates['$set'] = {}
            updates['$set'] = {'display_name': display_name}

        if phenotype_groups:
            if group_abbreviations:
                group_abbreviations = list(group_abbreviations)
            existing_groups = {}
            if add_groups:
                existing_groups = institute_obj.get('phenotype_groups', PHENOTYPE_GROUPS)

            for i,hpo_term in enumerate(phenotype_groups):
                hpo_obj = self.hpo_term(hpo_term)
                if not hpo_obj:
                    raise IntegrityError("Term {} does not exist".format(hpo_term))
                hpo_id = hpo_obj['hpo_id']
                description = hpo_obj['description']
                abbreviation = None
                if group_abbreviations:
                    abbreviation = group_abbreviations[i]
                existing_groups[hpo_term] = {'name': description, 'abbr':abbreviation}
            updates['$set'] = {'phenotype_groups': existing_groups}
        
        if updates:
            if not '$set' in updates:
                updates['$set'] = {}
            
            updates['$set']['updated_at'] = datetime.now()
            
            updated_institute = self.institute_collection.find_one_and_update(
                {'_id':internal_id}, updates, return_document = pymongo.ReturnDocument.AFTER)
            
            LOG.info("Institute updated")
        
        return updated_institute

    def institute(self, institute_id):
        """Featch a single institute from the backend

            Args:
                institute_id(str)

            Returns:
                Institute object
        """
        LOG.debug("Fetch institute {}".format(institute_id))
        institute_obj = self.institute_collection.find_one({
            '_id': institute_id
        })
        if institute_obj is None:
            LOG.debug("Could not find institute {0}".format(institute_id))

        return institute_obj

    def institutes(self, institute_ids=None):
        """Fetch all institutes.
        
        Args:
            institute_ids(list(str))
        
        Returns:
            res(pymongo.Cursor)
        """
        query = {}
        if institute_ids:
            query['_id'] = {'$in': institute_ids}
        LOG.debug("Fetching all institutes")
        return self.institute_collection.find(query)




import logging

import click

LOG = logging.getLogger(__name__)

def abort_if_false(ctx, param, value):
    if not value:
        ctx.abort()

@click.command('index', short_help='Index the database')
@click.option('--yes', is_flag=True, 
    callback=abort_if_false,
    expose_value=False,
    prompt="This will delete and rebuild all indexes(if not --update). Are you sure?"
)
@click.option('--update', help="Update the indexes", is_flag=True)
@click.pass_context
def index(context, update):
    """Create indexes for the database"""
    LOG.info("Running scout index")
    adapter = context.obj['adapter']
    
    if update:
        adapter.update_indexes()
    else:
        adapter.load_indexes()



import logging

from .transcript import build_transcript

from scout.constants import (CONSEQUENCE, FEATURE_TYPES, SO_TERM_KEYS)

LOG = logging.getLogger(__name__)

def build_gene(gene, hgncid_to_gene=None):
    """Build a gene object
        
        Has to build the transcripts for the genes to
    
    Args:
        gene(dict): Parsed information from the VCF
        hgncid_to_gene(dict): A map from hgnc_id  -> hgnc_gene objects 

    Returns:
        gene_obj(dict)
    
    gene = dict(
        # The hgnc gene id
        hgnc_id = int, # required
        hgnc_symbol = str, 
        ensembl_gene_id = str, 
        # A list of Transcript objects
        transcripts = list, # list of <transcript>
        # This is the worst functional impact of all transcripts
        functional_annotation = str, # choices=SO_TERM_KEYS
        # This is the region of the most severe functional impact
        region_annotation = str, # choices=FEATURE_TYPES
        # This is most severe sift prediction of all transcripts
        sift_prediction = str, # choices=CONSEQUENCE
        # This is most severe polyphen prediction of all transcripts
        polyphen_prediction = str, # choices=CONSEQUENCE
    )
    
    """
    hgncid_to_gene = hgncid_to_gene or {}
    gene_obj = dict()
 
    # This id is collected from the VCF
    # Typically annotated by VEP or snpEFF
    hgnc_id = int(gene['hgnc_id'])
    gene_obj['hgnc_id'] = hgnc_id

    # Get the gene information from database
    hgnc_gene = hgncid_to_gene.get(hgnc_id)
    
    inheritance = set()
    hgnc_transcripts = []
    if hgnc_gene:
        # Set the hgnc symbol etc to the one internally in Scout
        gene_obj['hgnc_symbol'] = hgnc_gene['hgnc_symbol']
        gene_obj['ensembl_id'] = hgnc_gene['ensembl_id']
        gene_obj['description'] = hgnc_gene['description']
        
        if hgnc_gene.get('ar'):
            inheritance.add('AR')
        if hgnc_gene.get('ad'):
            inheritance.add('AD')
        if hgnc_gene.get('xd'):
            inheritance.add('XD')
            inheritance.add('X')
        if hgnc_gene.get('xr'):
            inheritance.add('XR')
            inheritance.add('X')
        if hgnc_gene.get('y'):
            inheritance.add('Y')
    
    gene_obj['inheritance'] = list(inheritance)
    
    transcripts = []
    for transcript in gene['transcripts']:
        transcript_obj = build_transcript(transcript)
        transcripts.append(transcript_obj)
    gene_obj['transcripts'] = transcripts

    functional_annotation = gene.get('most_severe_consequence')
    if functional_annotation:
        if not functional_annotation in SO_TERM_KEYS:
            LOG.warning("Invalid functional annotation %s", functional_annotation)
        else:
            gene_obj['functional_annotation'] = functional_annotation
    
    region_annotation = gene.get('region_annotation')
    if region_annotation:
        if not region_annotation in FEATURE_TYPES:
            LOG.warning("Invalid region annotation %s", region_annotation)
        else:
            gene_obj['region_annotation'] = region_annotation
 
    sift_prediction = gene.get('most_severe_sift')
    if sift_prediction:
        if not sift_prediction in CONSEQUENCE:
            LOG.warning("Invalid sift prediction %s", sift_prediction)
        else:
            gene_obj['sift_prediction'] = sift_prediction

    polyphen_prediction = gene.get('most_severe_polyphen')
    if polyphen_prediction:
        if not polyphen_prediction in CONSEQUENCE:
            LOG.warning("Invalid polyphen prediction %s", polyphen_prediction)
        else:
            gene_obj['polyphen_prediction'] = polyphen_prediction
    
    gene_obj['hgvs_identifier'] = gene['hgvs_identifier']
    gene_obj['canonical_transcript'] = gene['canonical_transcript']
    gene_obj['exon'] = gene['exon']
    
    return gene_obj


# -*- coding: utf-8 -*-
from scout.build import build_individual
from scout.exceptions import PedigreeError
import pytest

def test_build_individual():
    ## GIVEN information about a individual
    ind_info = {
        'individual_id': '1',
        'father': '2',
        'mother': '3',
        'display_name': '1-1',
        'sex': 'male',
        'phenotype': 'affected',
        'bam_file': 'a.bam',
        'capture_kits': ['Agilent']
    }
    
    ## WHEN parsing the information
    ind_obj = build_individual(ind_info)

    ## THEN assert that information was parsed in the correct way
    assert ind_obj['individual_id'] == ind_info['individual_id']
    assert ind_obj['display_name'] == ind_info['display_name']
    assert ind_obj['capture_kits'] == ind_info['capture_kits']
    assert ind_obj['mother'] == ind_info['mother']
    assert ind_obj['sex'] == '1'
    assert ind_obj['phenotype'] == 2
    assert ind_obj['bam_file'] == ind_info['bam_file']
    assert ind_obj['analysis_type'] == 'unknown'

def test_build_individual_no_individual_id():
    ## GIVEN information about a individual without individual_id
    ind_info = {
        'father': '2',
        'mother': '3',
        'display_name': '1-1',
        'sex': 'male',
        'phenotype': 'affected',
        'bam_file': 'a.bam',
        'capture_kits': ['Agilent']
    }
    ## WHEN parsing the information
    ## THEN assert exception is raised since individual_id is required
    with pytest.raises(PedigreeError):
        ind_obj = build_individual(ind_info)

def test_build_individual_no_display_name():
    ## GIVEN information about a individual without display_name
    ind_info = {
        'individual_id': '1',
        'father': '2',
        'mother': '3',
        'sex': 'male',
        'phenotype': 'affected',
        'bam_file': 'a.bam',
        'capture_kits': ['Agilent']
    }
    ## WHEN parsing the information
    ind_obj = build_individual(ind_info)
    ## THEN assert that individual_id was used as display name
    assert ind_obj['display_name'] == ind_info['individual_id']

def test_build_individual_no_sex():
    
    ## GIVEN information about a individual without sex
    ind_info = {
        'individual_id': '1',
        'father': '2',
        'mother': '3',
        'phenotype': 'affected',
        'bam_file': 'a.bam',
        'capture_kits': ['Agilent']
    }
    ## WHEN parsing the information
    ind_obj = build_individual(ind_info)
    ## THEN assert that sex was set to other
    assert ind_obj['sex'] == '0'


def test_build_individual_wrong_sex():
    ## GIVEN information about a individual with malformed sex information
    ind_info = {
        'individual_id': '1',
        'father': '2',
        'mother': '3',
        'sex': 'random',
        'phenotype': 'affected',
        'bam_file': 'a.bam',
        'capture_kits': ['Agilent']
    }
    ## WHEN parsing the information
    ## THEN assert exception is raised since we can not determin sex
    with pytest.raises(PedigreeError):
        ind_obj = build_individual(ind_info)

def test_build_individual_no_phenotype():
    ## GIVEN information about a individual without phenotype
    ind_info = {
        'individual_id': '1',
        'father': '2',
        'mother': '3',
        'sex': 'male',
        'bam_file': 'a.bam',
        'capture_kits': ['Agilent']
    }
    ## WHEN parsing the information
    ind_obj = build_individual(ind_info)
    ## THEN assert phenitype was set to unknown (0)
    assert ind_obj['phenotype'] == 0

def test_build_individual_wrong_phenotype():
    ## GIVEN information about a individual with malformed phenotype information
    ind_info = {
        'individual_id': '1',
        'father': '2',
        'mother': '3',
        'sex': 'male',
        'phenotype': 'scrabble',
        'bam_file': 'a.bam',
        'capture_kits': ['Agilent']
    }
    ## WHEN parsing the information
    ## THEN assert exception is raised since we can not determin phenotype
    with pytest.raises(PedigreeError):
        ind_obj = build_individual(ind_info)

def test_build_individual_no_analysis_type():
    ## GIVEN information about a individual without analysis type
    ind_info = {
        'individual_id': '1',
    }
    ## WHEN parsing the information
    ind_obj = build_individual(ind_info)
    ## THEN assert analysis type was set to unknown
    assert ind_obj['analysis_type'] == 'unknown'

def test_build_individual_wrong_analysis_type():
    ## GIVEN information about a individual with malformed analysis type
    ind_info = {
        'individual_id': '1',
        'analysis_type': 'hello'
    }
    ## WHEN parsing the information
    ## THEN assert exception is raised since we can not determin analysis type
    with pytest.raises(PedigreeError):
        ind_obj = build_individual(ind_info)

def test_build_individuals(parsed_case):
    ## GIVEN a case with multiple individuals
    for ind_info in parsed_case['individuals']:
        ## WHEN building the ind_objs
        ind_obj = build_individual(ind_info)
        ## THEN assert they succeded
        assert ind_obj['individual_id'] == ind_info['individual_id']
        assert ind_obj['display_name'] == ind_info['display_name']
        assert ind_obj['capture_kits'] == ind_info['capture_kits']



import datetime

from scout.build.user import build_user

def test_build_user(parsed_user):
    ## GIVEN some parsed user information
    
    ## WHEN building a user object
    user_obj = build_user(parsed_user)
    
    ## THEN assert that the build was correct
    assert user_obj['_id'] == user_obj['email'] == parsed_user['email']

    assert user_obj['name'] == user_obj['name']
    
    assert isinstance(user_obj['created_at'], datetime.datetime)


# -*- coding: utf-8 -*-
import logging

import pytest
import pymongo

from scout.server.app import create_app
from scout.adapter import MongoAdapter
from scout.load.hgnc_gene import load_hgnc_genes
from scout.load.hpo import load_hpo


log = logging.getLogger(__name__)


class MockMail:
    _send_was_called = False
    _message = None

    def send(self, message):
        self._send_was_called = True
        self._message = message

@pytest.fixture
def mock_mail():
    return MockMail()

@pytest.fixture
def mock_sender():
    return 'mock_sender'



import logging

import click

logger = logging.getLogger(__name__)

from scout.load import load_institute


@click.command('institute', short_help='Load a institute')
@click.option('-i', '--internal-id',
              required=True)
@click.option('-d', '--display-name')
@click.option('-s', '--sanger-recipients', multiple=True)
@click.pass_context
def institute(ctx, internal_id, display_name, sanger_recipients):
    """
    Create a new institute and add it to the database

    """
    adapter = ctx.obj['adapter']

    if not internal_id:
        logger.warning("A institute has to have an internal id")
        ctx.abort()

    if not display_name:
        display_name = internal_id

    if sanger_recipients:
        sanger_recipients = list(sanger_recipients)

    try:
        load_institute(
            adapter=adapter,
            internal_id=internal_id,
            display_name=display_name,
            sanger_recipients=sanger_recipients
        )
    except Exception as e:
        logger.warning(e)
        ctx.abort()



# -*- coding: utf-8 -*-
import logging
import os.path

from flask import (abort, Blueprint, render_template, request, current_app, flash)

from .partial import send_file_partial

alignviewers_bp = Blueprint('alignviewers', __name__, template_folder='templates',
                      static_folder='static', static_url_path='/alignviewers/static')

LOG = logging.getLogger(__name__)


@alignviewers_bp.route('/remote/static', methods=['OPTIONS', 'GET'])
def remote_static():
    """Stream *large* static files with special requirements."""
    file_path = request.args.get('file')

    range_header = request.headers.get('Range', None)
    if not range_header and file_path.endswith('.bam'):
        return abort(500)

    new_resp = send_file_partial(file_path)
    return new_resp


@alignviewers_bp.route('/pileup')
def pileup():
    """Visualize BAM alignments."""
    vcf_file = request.args.get('vcf')
    bam_files = request.args.getlist('bam')
    bai_files = request.args.getlist('bai')
    samples = request.args.getlist('sample')
    alignments = [{'bam': bam, 'bai': bai, 'sample': sample}
                  for bam, bai, sample in zip(bam_files, bai_files, samples)]

    position = {
        'contig': request.args['contig'],
        'start': request.args['start'],
        'stop': request.args['stop']
    }

    genome = current_app.config.get('PILEUP_GENOME')
    if genome:
        if not os.path.isfile(genome):
            flash("The pilup genome path ({}) provided does not exist".format(genome))
            genome = None
    LOG.debug("Use pileup genome %s", genome)

    exons = current_app.config.get('PILEUP_EXONS')
    if exons:
        if not os.path.isfile(exons):
            flash("The pilup exons path ({}) provided does not exist".format(exons))
            genome = None
    LOG.debug("Use pileup exons %s", exons)

    LOG.debug("View alignment for positions Chrom:{0}, Start:{1}, End: {2}".format(
              position['contig'], position['start'], position['stop']))
    LOG.debug("Use alignment files {}".format(alignments))

    return render_template('alignviewers/pileup.html', alignments=alignments,
                           position=position, vcf_file=vcf_file,
                           genome=genome, exons=exons)


@alignviewers_bp.route('/igv')
def igv():
    """Visualize BAM alignments using igv.js (https://github.com/igvteam/igv.js)"""
    chrom = request.args.get('contig')
    if chrom == 'MT':
        chrom = 'M'

    start = request.args.get('start')
    stop = request.args.get('stop')

    locus = "chr{0}:{1}-{2}".format(chrom,start,stop)
    LOG.debug('Displaying locus %s', locus)

    chromosome_build = request.args.get('build')
    LOG.debug('Chromosome build is %s', chromosome_build)

    samples = request.args.getlist('sample')
    bam_files = request.args.getlist('bam')
    bai_files = request.args.getlist('bai')
    LOG.debug('loading the following tracks: %s', bam_files)

    display_obj={}

    # Add chromosome build info to the track object
    fastaURL = ''
    indexURL = ''
    cytobandURL = ''
    gene_track_format = ''
    gene_track_URL = ''
    gene_track_indexURL = ''

    if chromosome_build == "GRCh38" or chrom == 'M':
        fastaURL = 'https://s3.amazonaws.com/igv.broadinstitute.org/genomes/seq/hg38/hg38.fa'
        indexURL = 'https://s3.amazonaws.com/igv.broadinstitute.org/genomes/seq/hg38/hg38.fa.fai'
        cytobandURL = 'https://s3.amazonaws.com/igv.broadinstitute.org/annotations/hg38/cytoBandIdeo.txt'
        gene_track_format = 'gtf'
        gene_track_URL = 'https://s3.amazonaws.com/igv.broadinstitute.org/annotations/hg38/genes/Homo_sapiens.GRCh38.80.sorted.gtf.gz'
        gene_track_indexURL = 'https://s3.amazonaws.com/igv.broadinstitute.org/annotations/hg38/genes/Homo_sapiens.GRCh38.80.sorted.gtf.gz.tbi'

    else:
        fastaURL = 'https://s3.amazonaws.com/igv.broadinstitute.org/genomes/seq/hg19/hg19.fasta'
        indexURL = 'https://s3.amazonaws.com/igv.broadinstitute.org/genomes/seq/hg19/hg19.fasta.fai'
        cytobandURL = 'https://s3.amazonaws.com/igv.broadinstitute.org/genomes/seq/hg19/cytoBand.txt'
        gene_track_format = 'bed'
        gene_track_URL = 'https://s3.amazonaws.com/igv.broadinstitute.org/annotations/hg19/genes/refGene.hg19.bed.gz'
        gene_track_indexURL = 'https://s3.amazonaws.com/igv.broadinstitute.org/annotations/hg19/genes/refGene.hg19.bed.gz.tbi'

    display_obj['reference_track'] = {
        'fastaURL' : fastaURL,
        'indexURL' : indexURL,
        'cytobandURL' : cytobandURL
    }

    display_obj['genes_track'] = {
        'name' : 'Genes',
        'type' : 'annotation',
        'format': gene_track_format,
        'sourceType': 'file',
        'url' : gene_track_URL,
        'indexURL' : gene_track_indexURL,
        'displayMode' : 'EXPANDED'
    }

    sample_tracks = []
    counter = 0
    for sample in samples:
        # some samples might not have an associated bam file, take care if this
        if bam_files[counter]:
            sample_tracks.append({ 'name' : sample, 'url' : bam_files[counter], 'indexURL' : bai_files[counter] })
        counter += 1

    display_obj['sample_tracks'] = sample_tracks

    if request.args.get('center_guide'):
        display_obj['display_center_guide'] = True
    else:
        display_obj['display_center_guide'] = False

    return render_template('alignviewers/igv_viewer.html', locus=locus, **display_obj )



import pkg_resources


###### Files ######

# Gene panel:
panel_file = 'demo/panel_1.txt'
madeline_file = 'demo/madeline.xml'

# Case info
ped_file = 'demo/643594.ped'
load_file = 'demo/643594.config.yaml'
clinical_snv_file = 'demo/643594.clinical.vcf.gz'
research_snv_file = 'demo/643594.research.vcf.gz'

ped_path = pkg_resources.resource_filename('scout', ped_file)
clinical_sv_file = 'demo/643594.clinical.SV.vcf.gz'
research_sv_file = 'demo/643594.research.SV.vcf.gz'

panel_path = pkg_resources.resource_filename('scout', panel_file)
madeline_path = pkg_resources.resource_filename('scout', madeline_file)
load_path = pkg_resources.resource_filename('scout', load_file)

clinical_snv_path = pkg_resources.resource_filename('scout', clinical_snv_file)
clinical_sv_path = pkg_resources.resource_filename('scout', clinical_sv_file)

research_snv_path = pkg_resources.resource_filename('scout', research_snv_file)
research_sv_path = pkg_resources.resource_filename('scout', research_sv_file)


from scout.models.hgnc_map import HgncTranscript

def build_transcript(transcript_info, build='37'):
    """Build a hgnc_transcript object

        Args:
            transcript_info(dict): Transcript information

        Returns:
            transcript_obj(HgncTranscript)
            {
                transcript_id: str, required
                hgnc_id: int, required
                build: str, required
                refseq_id: str,
                chrom: str, required
                start: int, required
                end: int, required
                is_primary: bool
            }
    """
    try:
        transcript_id = transcript_info['ensembl_transcript_id']
    except KeyError:
        raise KeyError("Transcript has to have ensembl id")
    
    build = build
    is_primary = transcript_info.get('is_primary', False)
    
    refseq_id = transcript_info.get('refseq_id')
    refseq_identifiers = transcript_info.get('refseq_identifiers')

    try:
        chrom = transcript_info['chrom']
    except KeyError:
        raise KeyError("Transcript has to have a chromosome")
    
    try:
        start = int(transcript_info['transcript_start'])
    except KeyError:
        raise KeyError("Transcript has to have start")
    except TypeError:
        raise TypeError("Transcript start has to be integer")

    try:
        end = int(transcript_info['transcript_end'])
    except KeyError:
        raise KeyError("Transcript has to have end")
    except TypeError:
        raise TypeError("Transcript end has to be integer")

    try:
        hgnc_id = int(transcript_info['hgnc_id'])
    except KeyError:
        raise KeyError("Transcript has to have a hgnc id")
    except TypeError:
        raise TypeError("hgnc id has to be integer")

    transcript_obj = HgncTranscript(
        transcript_id=transcript_id, 
        hgnc_id=hgnc_id, 
        chrom=chrom, 
        start=start, 
        end=end, 
        is_primary=is_primary, 
        refseq_id=refseq_id,
        refseq_identifiers=refseq_identifiers,
        build=build
    )
    # Remove unnessesary keys
    for key in list(transcript_obj):
        if transcript_obj[key] is None:
            transcript_obj.pop(key)

    return transcript_obj



from scout.parse.variant.ids import (parse_ids, parse_simple_id, parse_variant_id,
                                     parse_display_name, parse_document_id)
from scout.utils.md5 import generate_md5_key


def test_parse_simple_id():
    # GIVEN some coordinates
    chrom = '1'
    pos = '10'
    ref = 'A'
    alt = 'G'

    # WHEN parsing the simple id
    simple_id = parse_simple_id(chrom, pos, ref, alt)
    # THEN we should get a correct simple id
    assert simple_id == '_'.join([chrom, pos, ref, alt])


def test_parse_variant_id():
    # GIVEN some coordinates
    chrom = '1'
    pos = '10'
    ref = 'A'
    alt = 'G'
    variant_type = 'clinical'

    # WHEN parsing the variant id
    variant_id = parse_variant_id(chrom, pos, ref, alt, variant_type)
    # THEN we should get a correct variant id
    assert variant_id == generate_md5_key([chrom, pos, ref, alt, variant_type])


def test_parse_display_name():
    # GIVEN some coordinates
    chrom = '1'
    pos = '10'
    ref = 'A'
    alt = 'G'
    variant_type = 'clinical'

    # WHEN parsing the display name
    variant_id = parse_display_name(chrom, pos, ref, alt, variant_type)
    # THEN we should get a correct display name
    assert variant_id == '_'.join([chrom, pos, ref, alt, variant_type])


def test_parse_document_id():
    # GIVEN some variant information
    chrom = '1'
    pos = '10'
    ref = 'A'
    alt = 'G'
    case_id = 'cust000_1'
    variant_type = 'clinical'

    # WHEN parsing the document id
    variant_id = parse_document_id(chrom, pos, ref, alt, variant_type, case_id)
    # THEN we should get a correct md5 string back
    assert variant_id == generate_md5_key([chrom, pos, ref, alt, variant_type, case_id])


def test_parse_ids():
    # GIVEN some variant and case information
    chrom = '1'
    pos = 10
    ref = 'A'
    alt = 'G'
    case_id = 'cust000_1'
    variant_type = 'clinical'

    # WHEN parsing the variant ids
    variant_ids = parse_ids(chrom, pos, ref, alt, case_id, variant_type)

    # THEN we should get a dictionary with all ids back
    assert isinstance(variant_ids, dict)
    assert variant_ids['simple_id'] == '_'.join([chrom, str(pos), ref, alt])



# -*- coding: utf-8 -*-
import logging
from datetime import datetime
from bson.objectid import ObjectId

import pymongo

LOG = logging.getLogger(__name__)

class ClinVarHandler(object):
    """Class to handle clinvar submissions for the mongo adapter"""

    def create_submission(self, user_id, institute_id):
        """Create an open clinvar submission for a user and an institute
           Args:
                user_id(str): a user ID
                institute_id(str): an institute ID

           returns:
                submission(obj): an open clinvar submission object
        """

        submission_obj = {
            'status' : 'open',
            'created_at' : datetime.now(),
            'user_id' : user_id,
            'institute_id' : institute_id
        }
        LOG.info("Creating a new clinvar submission for user '%s' and institute %s", user_id, institute_id)
        result = self.clinvar_submission_collection.insert_one(submission_obj)
        return result.inserted_id


    def delete_submission(self, submission_id):
        """Deletes a Clinvar submission object, along with all associated clinvar objects (variants and casedata)

            Args:
                submission_id(str): the ID of the submission to be deleted

            Returns:
                deleted_objects(int): the number of associated objects removed (variants and/or casedata)
                deleted_submissions(int): 1 if it's deleted, 0 if something went wrong
        """
        LOG.info("Deleting clinvar submission %s", submission_id)
        submission_obj = self.clinvar_submission_collection.find_one({ '_id' : ObjectId(submission_id)})

        submission_variants = submission_obj.get('variant_data')
        submission_casedata = submission_obj.get('case_data')

        submission_objects = []

        if submission_variants and submission_casedata:
            submission_objects = submission_variants + submission_casedata
        elif submission_variants:
            submission_objects = submission_variants
        elif submission_casedata:
            submission_objects = submission_casedata

        # Delete all variants and casedata objects associated with this submission
        result = self.clinvar_collection.delete_many({'_id': { "$in": submission_objects} })
        deleted_objects = result.deleted_count

        # Delete the submission itself
        result = self.clinvar_submission_collection.delete_one({'_id': ObjectId(submission_id)})
        deleted_submissions = result.deleted_count

        #return deleted_count, deleted_submissions
        return deleted_objects,deleted_submissions


    def get_open_clinvar_submission(self, user_id, institute_id):
        """Retrieve the database id of an open clinvar submission for a user and institute,
           if none is available then create a new submission and return it

           Args:
                user_id(str): a user ID
                institute_id(str): an institute ID

           Returns:
                submission(obj) : an open clinvar submission object
        """

        LOG.info("Retrieving an open clinvar submission for user '%s' and institute %s", user_id, institute_id)
        query = dict(user_id=user_id, institute_id=institute_id, status='open')
        submission = self.clinvar_submission_collection.find_one(query)

        # If there is no open submission for this user and institute, create one
        if submission is None:
            submission_id = self.create_submission(user_id, institute_id)
            submission = self.clinvar_submission_collection.find_one({'_id':submission_id})

        return submission


    def update_clinvar_id(self, clinvar_id, submission_id ):
        """saves an official clinvar submission ID in a clinvar submission object

            Args:
                clinvar_id(str): a string with a format: SUB[0-9]. It is obtained from clinvar portal when starting a new submission
                submission_id(str): submission_id(str) : id of the submission to be updated

            Returns:
                updated_submission(obj): a clinvar submission object, updated
        """
        updated_submission = self.clinvar_submission_collection.find_one_and_update( {'_id': ObjectId(submission_id)}, { '$set' : {'clinvar_subm_id' : clinvar_id, 'updated_at': datetime.now()} }, upsert=True, return_document=pymongo.ReturnDocument.AFTER )
        return updated_submission


    def get_clinvar_id(self, submission_id):
        """Returns the official Clinvar submission ID for a submission object

            Args:
                submission_id(str): submission_id(str) : id of the submission

            Returns:
                clinvar_subm_id(str): a string with a format: SUB[0-9]. It is obtained from clinvar portal when starting a new submission

        """
        submission_obj = self.clinvar_submission_collection.find_one({'_id': ObjectId(submission_id)})
        clinvar_subm_id = submission_obj.get('clinvar_subm_id') # This key does not exist if it was not previously provided by user
        return clinvar_subm_id


    def add_to_submission(self, submission_id, submission_objects):
        """Adds submission_objects to clinvar collection and update the coresponding submission object with their id

            Args:
                submission_id(str) : id of the submission to be updated
                submission_objects(tuple): a tuple of 2 elements coresponding to a list of variants and a list of case data objects to add to submission

            Returns:
                updated_submission(obj): an open clinvar submission object, updated
        """

        LOG.info("Adding new variants and case data to clinvar submission '%s'", submission_id)

        # Insert variant submission_objects into clinvar collection
        # Loop over the objects
        for var_obj in submission_objects[0]:
            try:
                result = self.clinvar_collection.insert_one(var_obj)
                self.clinvar_submission_collection.update_one({'_id':submission_id}, {'$push': { 'variant_data' : str(result.inserted_id) }}, upsert=True)
            except pymongo.errors.DuplicateKeyError:
                LOG.error("Attepted to insert a clinvar variant which is already in DB!")

        # Insert casedata submission_objects into clinvar collection
        if submission_objects[1]:
            # Loop over the objects
            for case_obj in submission_objects[1]:
                try:
                    result = self.clinvar_collection.insert_one(case_obj)
                    self.clinvar_submission_collection.update_one({'_id':submission_id}, {'$push': { 'case_data': str(result.inserted_id)}}, upsert=True)
                except pymongo.errors.DuplicateKeyError:
                    LOG.error("One or more casedata object is already present in clinvar collection!")

        updated_submission = self.clinvar_submission_collection.find_one_and_update( {'_id':submission_id}, { '$set' : {'updated_at': datetime.now()} }, return_document=pymongo.ReturnDocument.AFTER )
        return updated_submission


    def update_clinvar_submission_status(self, user_id, submission_id, status):
        """Set a clinvar submission ID to 'closed'

            Args:
                submission_id(str): the ID of the clinvar submission to close

            Return
                updated_submission(obj): the submission object with a 'closed' status

        """
        LOG.info('closing clinvar submission "%s"', submission_id)

        if status == 'open': # just close the submission its status does not affect the other submissions for this user
            # Close all other submissions for this user and then open the desired one
            self.clinvar_submission_collection.update_many(
                {'user_id' : user_id},
                {'$set' :
                    {'status' : 'closed', 'updated_at' : datetime.now()}
                }
            )
        updated_submission = self.clinvar_submission_collection.find_one_and_update(
            {'_id'  : ObjectId(submission_id)},
            {'$set' :
                {'status' : status, 'updated_at' : datetime.now()}
            },
            return_document=pymongo.ReturnDocument.AFTER
        )

        return updated_submission


    def clinvar_submissions(self, user_id, institute_id):
        """Collect all open and closed clinvar submission created by a user for an institute

            Args:
                user_id(str): a user ID
                institute_id(str): an institute ID

            Returns:
                submissions(list): a list of clinvar submission objects
        """
        LOG.info("Retrieving all clinvar submissions for user '%s', institute '%s'", user_id, institute_id)
        # get first all submission objects
        query = dict(user_id=user_id, institute_id=institute_id)
        results = list(self.clinvar_submission_collection.find(query))

        submissions = []
        for result in results:
            submission = {}
            submission['_id'] =  result.get('_id')
            submission['status'] = result.get('status')
            submission['user_id'] = result.get('user_id')
            submission['institute_id'] = result.get('institute_id')
            submission['created_at'] = result.get('created_at')
            submission['updated_at'] = result.get('updated_at')

            if 'clinvar_subm_id' in result:
                submission['clinvar_subm_id'] = result['clinvar_subm_id']

            if result.get('variant_data'):
                submission['variant_data'] = self.clinvar_collection.find({'_id': { "$in": result['variant_data'] } })
            if result.get('case_data'):
                submission['case_data'] = self.clinvar_collection.find({'_id' : { "$in": result['case_data'] } })

            submissions.append(submission)

        return submissions


    def clinvar_objs(self, submission_id, key_id):
        """Collects a list of objects from the clinvar collection (variants of case data) as specified by the key_id in the clinvar submission

            Args:
                submission_id(str): the _id key of a clinvar submission
                key_id(str) : either 'variant_data' or 'case_data'. It's a key in a clinvar_submission object.
                              Its value is a list of ids of clinvar objects (either variants of casedata objects)

            Returns:
                clinvar_objects(list) : a list of clinvar objects (either variants of casedata)

        """
        # Get a submission object
        submission = self.clinvar_submission_collection.find_one({'_id': ObjectId(submission_id)})

        # a list of clinvar object ids, they can be of csv_type 'variant' or 'casedata'
        if submission.get(key_id):
            clinvar_obj_ids = list(submission.get(key_id))
            clinvar_objects = self.clinvar_collection.find({'_id' : { "$in": clinvar_obj_ids }})
            return list(clinvar_objects)
        else:
            return None


    def delete_clinvar_object(self, object_id, object_type, submission_id):
        """Remove a variant object from clinvar database and update the relative submission object

            Args:
                object_id(str) : the id of an object to remove from clinvar_collection database collection (a variant of a case)
                object_type(str) : either 'variant_data' or 'case_data'. It's a key in the clinvar_submission object.
                submission_id(str): the _id key of a clinvar submission

            Returns:
                updated_submission(obj): an updated clinvar submission
        """

        LOG.info("Deleting clinvar object %s (%s)", object_id, object_type)

        # If it's a variant object to be removed:
        #   remove reference to it in the submission object 'variant_data' list field
        #   remove the variant object from clinvar collection
        #   remove casedata object from clinvar collection
        #   remove reference to it in the submission object 'caset_data' list field

        # if it's a casedata object to be removed:
        #   remove reference to it in the submission object 'caset_data' list field
        #   remove casedata object from clinvar collection

        result = ''

        if object_type == 'variant_data':
            # pull out a variant from submission object
            self.clinvar_submission_collection.find_one_and_update( {'_id': ObjectId(submission_id)}, {'$pull': {'variant_data': object_id} })

            variant_object = self.clinvar_collection.find_one( {'_id': object_id} )
            linking_id = variant_object.get("linking_id") #it's the original ID of the variant in scout, it's linking clinvar variants and casedata objects together

            # remove any object with that linking_id from clinvar_collection. This removes variant and casedata
            result = self.clinvar_collection.delete_many( {'linking_id': linking_id } )

        else: # remove case_data but keep variant in submission
            # delete the object itself from clinvar_collection
             result = self.clinvar_collection.delete_one( {'_id': object_id } )

        # in any case remove reference to it in the submission object 'caset_data' list field
        self.clinvar_submission_collection.find_one_and_update( {'_id': ObjectId(submission_id)}, {'$pull': {'case_data': object_id} })

        updated_submission = self.clinvar_submission_collection.find_one_and_update( {'_id':submission_id}, { '$set' : {'updated_at': datetime.now()} }, return_document=pymongo.ReturnDocument.AFTER )

        return updated_submission


    def case_to_clinVars(self, case_id):
        """Get all variants included in clinvar submissions for a case

        Args:
            case_id(str): a case display_name

        Returns:
            submission_variants(dict): keys are variant ids and values are variant submission objects

        """
        query = dict(case_id=case_id, csv_type='variant')
        clinvar_objs = list(self.clinvar_collection.find(query))
        submitted_vars = {}
        for clinvar in clinvar_objs:
            submitted_vars[clinvar.get('local_id')] = clinvar

        return submitted_vars



import pkg_resources

###### Files ######

# Gene sources:
hgnc_file = 'demo/resources/hgnc_reduced_set.txt'
exac_file = 'demo/resources/forweb_cleaned_exac_r03_march16_z_data_pLI_reduced.txt'
transcripts37_file = 'demo/resources/ensembl_transcripts_37_reduced.txt'
genes37_file = 'demo/resources/ensembl_genes_37_reduced.txt'
transcripts38_file = 'demo/resources/ensembl_transcripts_38_reduced.txt'
genes38_file = 'demo/resources/ensembl_genes_38_reduced.txt'

# OMIM resources:
mim2gene_file = 'demo/resources/mim2gene_reduced.txt'
genemap2_file = 'demo/resources/genemap2_reduced.txt'

# Hpo resources
hpogenes_file = 'demo/resources/ALL_SOURCES_ALL_FREQUENCIES_genes_to_phenotype_reduced.txt'
hpoterms_file = 'demo/resources/reduced.hpo.obo'
hpo_phenotype_to_terms = 'demo/resources/ALL_SOURCES_ALL_FREQUENCIES_diseases_to_genes_to_phenotypes_reduced.txt'
hpo_to_genes = 'demo/resources/ALL_SOURCES_ALL_FREQUENCIES_phenotype_to_genes_reduced.txt'

hpo_terms_def_file = 'demo/resources/hpo_terms.csv'

# Additional resources
madeline_file = 'demo/madeline.xml'

###### Paths ######

# Gene paths
hgnc_reduced_path = pkg_resources.resource_filename('scout', hgnc_file)
exac_reduced_path = pkg_resources.resource_filename('scout', exac_file)
transcripts37_reduced_path = pkg_resources.resource_filename('scout', transcripts37_file)
transcripts38_reduced_path = pkg_resources.resource_filename('scout', transcripts38_file)
genes37_reduced_path = pkg_resources.resource_filename('scout', genes37_file)
genes38_reduced_path = pkg_resources.resource_filename('scout', genes38_file)

# OMIM paths
mim2gene_reduced_path = pkg_resources.resource_filename('scout', mim2gene_file)
genemap2_reduced_path = pkg_resources.resource_filename('scout', genemap2_file)

# Hpo paths
hpogenes_reduced_path = pkg_resources.resource_filename('scout', hpogenes_file)
hpoterms_reduced_path = pkg_resources.resource_filename('scout', hpoterms_file)
hpo_phenotype_to_terms_reduced_path = pkg_resources.resource_filename('scout', hpo_phenotype_to_terms)
hpo_to_genes_reduced_path = pkg_resources.resource_filename('scout', hpo_to_genes)

hpo_terms_def_path = pkg_resources.resource_filename('scout', hpo_terms_def_file)


# Additional paths
madeline_path = pkg_resources.resource_filename('scout', madeline_file)



# -*- coding: utf-8 -*-
from collections import OrderedDict

# from worst to most certain benign
ACMG_MAP = OrderedDict([
    (4, 'pathogenic'),
    (3, 'likely_pathogenic'),
    (0, 'uncertain_significance'),
    (2, 'likely_benign'),
    (1, 'benign'),
])

REV_ACMG_MAP = OrderedDict([(value, key) for key, value in ACMG_MAP.items()])

ACMG_OPTIONS = [{
    'code': 'pathogenic',
    'short': 'P',
    'label': 'Pathogenic',
    'color': 'danger',
}, {
    'code': 'likely_pathogenic',
    'short': 'LP',
    'label': 'Likely Pathogenic',
    'color': 'warning',
}, {
    'code': 'uncertain_significance',
    'short': 'VUS',
    'label': 'Uncertain Significance',
    'color': 'primary',
}, {
    'code': 'likely_benign',
    'short': 'LB',
    'label': 'Likely Benign',
    'color': 'info',
}, {
    'code': 'benign',
    'short': 'B',
    'label': 'Benign',
    'color': 'success',
}]

ACMG_COMPLETE_MAP = OrderedDict([(option['code'], option) for option in ACMG_OPTIONS])

ACMG_CRITERIA = OrderedDict()

ACMG_CRITERIA['pathogenicity'] = OrderedDict([
    ('Very Strong', OrderedDict([
        ('PVS1', {
            'short': 'Null variant',
            'description': 'Null variant (nonsense, frameshift, canonical +/- 2 bp splice sites, initiation codon, single or multiexon deletion) in a gene where LOF is a known mechanism of disease.',
        }),
    ])),
    ('Strong', OrderedDict([
        ('PS1', {
            'short': 'Known pathogenic aa (HQ)',
            'description': 'Same amino acid change as a previously established pathogenic variant regardless of nucleotide change',
        }),
        ('PS2', {
            "short": "De novo (confirmed)",
            "description": "De novo (both maternity and paternity confirmed) in a patient with the disease and no family history"
        }),
        ('PS3', {
            "short": "Functional damage (HQ)",
            "description": "Well-established in vitro or in vivo functional studies supportive of a damaging effect on the gene or gene product, and the evidence is strong"
        }),
        ('PS4', {
            "short": "In >=4 unrelated patients, not controls",
            "description": "The prevalence of the variant in affected individuals is significantly increased compared with the prevalence in controls; 4 or more unrelated patients"
        }),
    ])),
    ('Moderate', OrderedDict([
        ('PM1', {
            "short": "Functional domain",
            "description": "Located in a mutational hot spot and/or critical and well-established functional domain  (e.g., active site of an enzyme) without benign variation"
        }),
        ('PM2', {
            "short": "Not in matched controls",
            "description": "Absent from controls (or at extremely low frequency if recessive), in ethnically matched population"
        }),
        ('PM3', {
            "short": "In trans pathogenic & AR",
            "description": "For recessive disorders, detected in trans with a pathogenic variant."
        }),
        ('PM4', {
            "short": "In-frame/stop-loss; moderate impact",
            "description": "Protein length changes as a result of in-frame deletions/insertions in a nonrepeat region or stop-loss variants."
        }),
        ('PM5', {
            "short": "Similar to known pathogenic aa (HQ)",
            "description": "Novel missense change at an amino acid residue where a different missense change determined to be pathogenic has been seen before,  the amino acids have similar properties and the evidence is strong"
        }),
        ('PM6', {
            "short": "De novo (unconfirmed)",
            "description": "Assumed de novo, but without confirmation of paternity and maternity"
        }),
    ])),
    ('Supporting', OrderedDict([
        ('PP1', {
            "short": "Cosegregation (WQ)",
            "description": "Cosegregation with disease in multiple affected family members in a gene definitively known to cause the disease, and the evidence is weak"
        }),
        ('PP2', {
            "short": "Missense: important",
            "description": "Missense variant in a gene that has a low rate of benign missense variation and in which missense variants are a common mechanism of disease."
        }),
        ('PP3', {
            "short": "Predicted pathogenic",
            "description": "Multiple lines of computational evidence support a deleterious effect on the gene or gene product (conservation, evolutionary, splicing impact, etc.)"
        }),
        ('PP4', {
            "short": "Phenotype: single gene",
            "description": "Patient's phenotype or family history is highly specific for a disease with a single genetic etiology"
        }),
        ('PP5', {
            "short": "Reported pathogenic, evidence unavailable",
            "description": "Reputable source recently reports variant as pathogenic, but the evidence is not available to the laboratory to perform an independent evaluation"
        }),
    ])),
])

ACMG_CRITERIA['benign impact'] = OrderedDict([
    ('Stand-alone', OrderedDict([
        ('BA1', {
            'short': 'Frequency >=hi_freq_cutoff',
            'description': 'Allele frequency is >=hi_freq_cutoff in ESP, 1000G or ExAC',
        }),
    ])),
    ('Strong', OrderedDict([
        ('BS1', {
            "short": "Frequency >expected & AD",
            "description": "Allele frequency is greater than expected for disorder, and the inheritance is autosomal dominant."
        }),
        ('BS2', {
            "short": "In documented healthy",
            "description": "Observed in a healthy adult individual for a recessive (homozygous), dominant (heterozygous), or X-linked (hemizygous) disorder, with full penentrance expected at an early age"
        }),
        ('BS3', {
            "short": "No functional damage (HQ)",
            "description": "Well-established in vitro or in vivo functional studies show no damaging effect on protein function or splicing, and the evidence is strong"
        }),
        ('BS4', {
            "short": "Non-segregation (HQ)",
            "description": "Lack of segregation in affected members of a family, and the evidence is strong"
        }),
    ])),
    ('Supporting', OrderedDict([
        ('BP1', {
            "short": "Missense; not important",
            "description": "Missense variant in a gene for which primarily truncating variants are known to cause disease"
        }),
        ('BP2', {
            "short": "In trans & AD, or in cis pathogenic",
            "description": "Observed in trans with a pathogenic variant for a fully penetrant dominant gene/disorder or in cis with a pathogenic variant in any inheritance pattern"
        }),
        ('BP3', {
            "short": "In-frame; non-functional",
            "description": "In-frame insertions/delitions in a repetitive region without a known function"
        }),
        ('BP4', {
            "short": "Predicted benign",
            "description": "Multiple lines of computational evidence suggest no impact on gene or gene product (conservation, evolutionary, splicing impact, etc.)"
        }),
        ('BP5', {
            "short": "Other causative variant found",
            "description": "Variant found in a case with an alternate molecular basis for disease"
        }),
        ('BP6', {
            "short": "Reported benign, evidence unavailable",
            "description": "Reputable source recently reports variant as pathogenic, but the evidence is not available to the laboratory to perform an independent evaluation"
        }),
        ('BP7', {
            "short": "(not in use)",
            "description": "A synonymous variant for which splicing prediction algorithms predict no impact to the splice consensus sequence nor the creation of a new splice site"
        }),
    ]))
])



# -*- coding: utf-8 -*-
import sys
import logging

from pprint import pprint as pp

import click
import coloredlogs
import yaml

# Adapter stuff
from scout.adapter.mongo import MongoAdapter
from scout.adapter.client import get_connection
from pymongo.errors import (ConnectionFailure, OperationFailure)

# General, logging
from scout import __version__

# Commands
from scout.commands.load import load as load_command
from scout.commands.export import export
from scout.commands.wipe_database import wipe
from scout.commands.setup import setup as setup_command
from scout.commands.convert import convert
from scout.commands.query import query as query_command
from scout.commands.view import view as view_command
from scout.commands.delete import delete
from scout.commands.serve import serve
from scout.commands.update import update as update_command
from scout.commands.index_command import index as index_command

from scout.adapter.utils import check_connection

try:
    from scoutconfig import *
except ImportError:
    pass

LOG_LEVELS = ['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL']
LOG = logging.getLogger(__name__)


@click.group()
@click.option('--loglevel', default='INFO', type=click.Choice(LOG_LEVELS),
              help="Set the level of log output.", show_default=True)
@click.option('-db', '--mongodb', help='Name of mongo database [scout]')
@click.option('-u', '--username')
@click.option('-p', '--password')
@click.option('-a', '--authdb', help='database to use for authentication')
@click.option('-port', '--port', help="Specify on what port to listen for the mongod")
@click.option('-h', '--host', help="Specify the host for the mongo database.")
@click.option('-c', '--config', type=click.Path(exists=True),
              help="Specify the path to a config file with database info.")
@click.option('--demo', is_flag=True, help="If the demo database should be used")
@click.version_option(__version__)
@click.pass_context
def cli(context, mongodb, username, password, authdb, host, port, loglevel, config, demo):
    """scout: manage interactions with a scout instance."""
    # log_format = "%(message)s" if sys.stdout.isatty() else None
    log_format = None
    coloredlogs.install(level=loglevel, fmt=log_format)
    LOG.info("Running scout version %s", __version__)
    LOG.debug("Debug logging enabled.")

    mongo_config = {}
    cli_config = {}
    if config:
        LOG.debug("Use config file %s", config)
        with open(config, 'r') as in_handle:
            cli_config = yaml.load(in_handle)

    mongo_config['mongodb'] = (mongodb or cli_config.get('mongodb') or 'scout')
    if demo:
        mongo_config['mongodb'] = 'scout-demo'

    mongo_config['host'] = (host or cli_config.get('host') or 'localhost')
    mongo_config['port'] = (port or cli_config.get('port') or 27017)
    mongo_config['username'] = username or cli_config.get('username')
    mongo_config['password'] = password or cli_config.get('password')
    mongo_config['authdb'] = authdb or cli_config.get('authdb') or mongo_config['mongodb']
    mongo_config['omim_api_key'] = cli_config.get('omim_api_key')

    if context.invoked_subcommand in ('setup', 'serve'):
        mongo_config['adapter'] = None
    else:
        LOG.info("Setting database name to %s", mongo_config['mongodb'])
        LOG.debug("Setting host to %s", mongo_config['host'])
        LOG.debug("Setting port to %s", mongo_config['port'])

        try:
            client = get_connection(**mongo_config)
        except ConnectionFailure:
            context.abort()

        database = client[mongo_config['mongodb']]

        LOG.info("Setting up a mongo adapter")
        mongo_config['client'] = client
        adapter = MongoAdapter(database)
        mongo_config['adapter'] = adapter
        
        LOG.info("Check if authenticated...")
        try:
            for ins_obj in adapter.institutes():
                pass
        except OperationFailure as err:
            LOG.info("User not authenticated")
            context.abort()
        

    context.obj = mongo_config


cli.add_command(load_command)
cli.add_command(wipe)
cli.add_command(setup_command)
cli.add_command(export)
cli.add_command(convert)
cli.add_command(query_command)
cli.add_command(view_command)
cli.add_command(delete)
cli.add_command(serve)
cli.add_command(update_command)
cli.add_command(index_command)



import logging

LOG = logging.getLogger(__name__)

def build_transcript(transcript, build='37'):
    """Build a transcript object
    
    These represents the transcripts that are parsed from the VCF, not
    the transcript definitions that are collected from ensembl.
    
        Args:
            transcript(dict): Parsed transcript information
    
        Returns:
            transcript_obj(dict)
    """
    
    # Transcripts has to have an id
    transcript_id = transcript['transcript_id']
    transcript_obj = dict(
        transcript_id = transcript_id
    )
    
    # Transcripts has to belong to a gene
    transcript_obj['hgnc_id'] = transcript['hgnc_id']

    if transcript.get('protein_id'):
        transcript_obj['protein_id'] = transcript['protein_id']
    
    if transcript.get('sift_prediction'):
        transcript_obj['sift_prediction'] = transcript['sift_prediction']
    
    if transcript.get('polyphen_prediction'):
        transcript_obj['polyphen_prediction'] = transcript['polyphen_prediction']


    if transcript.get('swiss_prot'):
        transcript_obj['swiss_prot'] = transcript['swiss_prot']
    
    if transcript.get('pfam_domain'):
        transcript_obj['pfam_domain'] = transcript.get('pfam_domain')
    
    if transcript.get('prosite_profile'):
        transcript_obj['prosite_profile'] = transcript.get('prosite_profile')
    
    if transcript.get('smart_domain'):
        transcript_obj['smart_domain'] = transcript.get('smart_domain')

    if transcript.get('biotype'):
        transcript_obj['biotype'] = transcript.get('biotype')

    if transcript.get('functional_annotations'):
        transcript_obj['functional_annotations'] = transcript['functional_annotations']
    
    if transcript.get('region_annotations'):
        transcript_obj['region_annotations'] = transcript['region_annotations']

    if transcript.get('exon'):
        transcript_obj['exon'] = transcript.get('exon')
    
    if transcript.get('intron'):
        transcript_obj['intron'] = transcript.get('intron')
    
    if transcript.get('strand'):
        transcript_obj['strand'] = transcript.get('strand')

    if transcript.get('coding_sequence_name'):
        transcript_obj['coding_sequence_name'] = transcript['coding_sequence_name']
    
    if transcript.get('protein_sequence_name'):
        transcript_obj['protein_sequence_name'] = transcript['protein_sequence_name']

    transcript_obj['is_canonical'] = transcript.get('is_canonical', False)

    return transcript_obj



def parse_cadd(variant, transcripts):
    """Check if the cadd phred score is annotated"""
    cadd = 0
    cadd_keys = ['CADD', 'CADD_PHRED']
    for key in cadd_keys:
        cadd = variant.INFO.get(key, 0)
        if cadd:
            return float(cadd)
    
    for transcript in transcripts:
        cadd_entry = transcript.get('cadd')
        if (cadd_entry and cadd_entry > cadd):
            cadd = cadd_entry
    
    return cadd
    


from scout.parse.variant.frequency import (parse_frequency, parse_frequencies, 
                                           parse_sv_frequencies)

def test_parse_frequency(cyvcf2_variant):
    # GIVEN a variant dict with a frequency in info_dict
    variant = cyvcf2_variant
    
    variant.INFO['1000G'] = 0.01
    
    # WHEN frequency is parsed
    frequency = parse_frequency(variant, '1000G')
    
    # THEN the frequency should be a float with correct value
    
    assert frequency == float(variant.INFO['1000G'])


def test_parse_frequency_non_existing_keyword(cyvcf2_variant):
    variant = cyvcf2_variant
    # GIVEN a variant dict with a frequency in info_dict    
    variant.INFO['1000G'] = 0.01
    
    # WHEN frequency is parsed with wrong keyword
    frequency = parse_frequency(variant, 'EXACAF')
    
    # THEN the frequency should be None
    assert frequency == None

def test_parse_frequency_non_existing_freq(cyvcf2_variant):
    variant = cyvcf2_variant
    # GIVEN a variant dict with a frequency in info_dict without value
    variant.INFO['1000G'] = None
    
    # WHEN frequency is parsed
    frequency = parse_frequency(variant, '1000G')

    # THEN the frequency should be None
    assert frequency == None

def test_parse_frequencies(cyvcf2_variant):
    variant = cyvcf2_variant
    # GIVEN a variant dict with a differenct frequencies
    variant.INFO['1000GAF'] = '0.01'
    variant.INFO['EXACAF'] = '0.001'
    
    # WHEN frequencies are parsed    
    frequencies = parse_frequencies(variant, [])
    
    # THEN the frequencies should be returned in a dictionary
    assert frequencies['thousand_g'] == float(variant.INFO['1000GAF'])
    assert frequencies['exac'] == float(variant.INFO['EXACAF'])

def test_parse_sv_frequencies_clingen_benign(cyvcf2_variant):
    variant = cyvcf2_variant
    # GIVEN a variant dict with a differenct frequencies
    variant.INFO['clingen_cgh_benignAF'] = '0.01'
    variant.INFO['clingen_cgh_benign'] = '3'
    
    # WHEN frequencies are parsed    
    frequencies = parse_sv_frequencies(variant)
    
    # THEN the frequencies should be returned in a dictionary
    assert frequencies['clingen_cgh_benignAF'] == float(variant.INFO['clingen_cgh_benignAF'])
    assert frequencies['clingen_cgh_benign'] == int(variant.INFO['clingen_cgh_benign'])



from pprint import pprint as pp

from scout.build.genes.hgnc_gene import build_hgnc_gene
import pytest

def test_build_hgnc_genes(genes):
    # GIVEN a dictionary with hgnc genes
    
    # WHEN building hgnc gene objecs
    for hgnc_id in genes:
        gene_info = genes[hgnc_id]
        gene_obj = build_hgnc_gene(gene_info)
        # THEN check that the gene models have a hgnc id
        assert gene_obj['hgnc_id']

def test_build_hgnc_gene():
    gene_info = {
        'hgnc_id': 100,
        'hgnc_symbol': 'TEST',
        'ensembl_gene_id': 'ENSTEST',
        'chromosome': '1',
        'start': 1,
        'end': 1000,
    }
    gene_obj = build_hgnc_gene(gene_info)
    
    assert gene_obj['hgnc_id'] == gene_info['hgnc_id']
    assert gene_obj['hgnc_symbol'] == gene_info['hgnc_symbol']
    assert gene_obj['length'] == gene_info['end'] - gene_info['start']
    assert gene_obj['ensembl_id'] == gene_info['ensembl_gene_id']

def test_build_hgnc_gene_no_id():
    gene_info = {
        'hgnc_symbol': 'TEST',
        'ensembl_gene_id': 'ENSTEST',
        'chromosome': '1',
        'start': 1,
        'end': 1000,
    }
    with pytest.raises(KeyError):
        gene_obj = build_hgnc_gene(gene_info)

def test_build_hgnc_gene_wrong_id():
    gene_info = {
        'hgnc_id': 'test',
        'hgnc_symbol': 'TEST',
        'ensembl_gene_id': 'ENSTEST',
        'chromosome': '1',
        'start': 1,
        'end': 1000,
    }
    with pytest.raises(ValueError):
        gene_obj = build_hgnc_gene(gene_info)



# -*- coding: utf-8 -*-
import os.path
import logging

import click
from livereload import Server

from scout.server.app import create_app

from scout.adapter.utils import check_connection

log = logging.getLogger(__name__)

@click.command()
@click.option('-c', '--config', type=click.Path(exists=True), envvar='SCOUT_CONFIG',
              help='Python config file')
@click.option('-h', '--host', default='localhost', help='Where to serve')
@click.option('-p', '--port', default=5000, help='Which port to listen on')
@click.option('-d', '--debug', is_flag=True, help='Run server in debug mode')
@click.option('-l', '--livereload', is_flag=True, help='Enable Live Reload server')
@click.pass_context
def serve(context, config, host, port, debug, livereload):
    """Start the web server."""
    pymongo_config = dict(
        MONGO_HOST=context.obj['host'],
        MONGO_PORT=context.obj['port'],
        MONGO_DBNAME=context.obj['mongodb'],
        MONGO_USERNAME=context.obj['username'],
        MONGO_PASSWORD=context.obj['password'],
    )
    
    valid_connection = check_connection(
        host=pymongo_config['MONGO_HOST'], 
        port=pymongo_config['MONGO_PORT'], 
        username=pymongo_config['MONGO_USERNAME'], 
        password=pymongo_config['MONGO_PASSWORD'],
        authdb=context.obj['authdb'],
        )

    log.info("Test if mongod is running")
    if not valid_connection:
        log.warning("Connection could not be established")
        log.info("Is mongod running?")
        context.abort()
    

    config = os.path.abspath(config) if config else None
    app = create_app(config=pymongo_config, config_file=config)
    if livereload:
        server = Server(app.wsgi_app)
        server.serve(host=host, port=port, debug=debug)
    else:
        app.run(host=host, port=port, debug=debug)



from scout.parse.variant.conservation import (parse_conservation, parse_conservations)

def test_parse_conservation(cyvcf2_variant):
    variant = cyvcf2_variant
    ## GIVEN a variant with some GERP information
    variant.INFO['GERP++_RS_prediction_term'] = 'Conserved'
    ## WHEN parsing conservation
    ## THEN assert that the field is parsed correct
    assert parse_conservation(variant, 'GERP++_RS_prediction_term') == ['Conserved']

def test_parse_conservation_wrong_term(cyvcf2_variant):
    variant = cyvcf2_variant
    ## GIVEN a variant with wrong GERP information
    variant.INFO['GERP++_RS_prediction_term'] = 'Conservation'
    ## WHEN parsing conservation
    ## THEN assert that nothing is returned
    assert parse_conservation(variant, 'GERP++_RS_prediction_term') == []

def test_parse_conservation_multiple_terms(cyvcf2_variant):
    variant = cyvcf2_variant
    ## GIVEN a variant with multiple GERP annotations
    variant.INFO['GERP++_RS_prediction_term'] = 'Conserved, NotConserved'

    ## WHEN parsing conservation
    ## THEN assert that all terms are returned
    assert parse_conservation(variant, 'GERP++_RS_prediction_term') == ['Conserved', 'NotConserved']

def test_parse_conservations(cyvcf2_variant):
    variant = cyvcf2_variant
    ## GIVEN a variant with multiple conservation annotations
    variant.INFO['GERP++_RS_prediction_term'] = 'Conserved,NotConserved'
    variant.INFO['phastCons100way_vertebrate_prediction_term'] = 'Conserved'
    variant.INFO['phyloP100way_vertebrate_prediction_term'] = 'NotConserved'

    ## WHEN parsing conservation
    ## THEN assert that all terms are returned

    conservations = parse_conservations(variant)
    
    assert conservations['gerp'] == ['Conserved', 'NotConserved']
    assert conservations['phast'] == ['Conserved']
    assert conservations['phylop'] == ['NotConserved']


# -*- coding: utf-8 -*-
import logging

import pytest
import pymongo

from scout.server.app import create_app
from scout.adapter import MongoAdapter
from scout.load.hgnc_gene import load_hgnc_genes
from scout.load.hpo import load_hpo

log = logging.getLogger(__name__)

@pytest.fixture
def database_name(request):
    db_name = 'realtestdb'
    return db_name


@pytest.fixture
def app(database_name, real_database):
    app = create_app(config=dict(TESTING=True, DEBUG=True, MONGO_DBNAME=database_name,
                                 DEBUG_TB_ENABLED=False, LOGIN_DISABLED=True))
    return app


@pytest.fixture
def institute_info():
    _institute_info = dict(internal_id='cust000', display_name='test_institute')
    return _institute_info


@pytest.fixture
def user_info(institute_info):
    _user_info = dict(email='john@doe.com', name='John Doe', roles=['admin'],
                      institutes=[institute_info['internal_id']])
    return _user_info


@pytest.fixture
def real_database(request, database_name, institute_obj, user_obj, genes, parsed_panel):
    """Setup a real database with populated data"""
    mongo_client = pymongo.MongoClient()
    mongo_client.drop_database(database_name)

    database = mongo_client[database_name]
    adapter = MongoAdapter(database)

    # Populate the database
    adapter.add_institute(institute_obj)
    adapter.add_user(user_obj)

    load_hgnc_genes(adapter, genes)
    adapter.hgnc_collection.create_index([('build', pymongo.ASCENDING),
                                          ('hgnc_symbol', pymongo.ASCENDING)])
    adapter.load_panel(parsed_panel=parsed_panel)

    # load_hpo(adapter=gene_database, hpo_lines=hpo_terms_handle, disease_lines=genemap_handle)
    # adapter.add_case(case_obj)

    # for variant in variant_objs:
    #     adapter.load_variant(variant)

    def teardown():
        mongo_client.drop_database(database_name)

    request.addfinalizer(teardown)

    return adapter



from scout.build.hpo import build_hpo_term

def test_build_hpo_term(adapter):
    ## GIVEN a hpo term
    hpo_info = {
        'hpo_id':"HP:0000878",
        'description': "11 pairs of ribs",
        'genes': [1, 2]
    }
    ## WHEN building the hpo term
    hpo_obj = build_hpo_term(hpo_info)
    ## THEN assert that the term has the correct information
    assert hpo_obj['_id'] == hpo_obj['hpo_id'] == hpo_info['hpo_id']    
    assert hpo_obj['description'] == hpo_info['description']
    assert len(hpo_obj['genes']) == 2



# -*- coding: utf-8 -*-

def hpo_terms(store, query = None, limit = None):
    """Retrieves a list of HPO terms from scout database

    Args:
        store (obj): an adapter to the scout database
        query (str): the term to search in the database
        limit (str): the number of desired results

    Returns:
        hpo_phenotypes (dict): the complete list of HPO objects stored in scout

    """
    hpo_phenotypes = {}
    if limit:
        limit=int(limit)

    hpo_phenotypes['phenotypes'] = list(store.hpo_terms(text=query, limit=limit))
    return hpo_phenotypes
